@@ -1,22 +1,43 @@
-<?php rcs_id('$Id: HtmlElement.php,v 1.31 2003/02/27 22:47:26 dairiki Exp $');
-/*
- * Code for writing XML.
+<?php rcs_id('$Id: HtmlElement.php,v 1.46 2005/01/25 06:50:33 rurban Exp $');
+/**
+ * Code for writing the HTML subset of XML.
+ * @author: Jeff Dairiki
+ *
+ * This code is now php5 compatible. --2004-04-19 23:51:43 rurban
  */
-require_once("lib/XmlElement.php");
+if (!class_exists("XmlElement"))
+    require_once(dirname(__FILE__)."/XmlElement.php");
+if (class_exists("HtmlElement"))
+    return;
+
 /**
  * An XML element.
  */
+//apd_set_session_trace(35);
 
 class HtmlElement extends XmlElement
 {
-    //function HtmlElement ($tagname /* , $attr_or_content , ...*/) {
-    //    $this->_init(func_get_args());
-    //    $this->_properties = HTML::getTagProperties($tagname);
-    //}
-
+    function __construct ($tagname /* , $attr_or_content , ...*/) {
+        $this->_init(func_get_args());
+        $this->_properties = HTML::getTagProperties($tagname);
+    }
 
     function _init ($args) {
-        XmlElement::_init($args);
+        if (!is_array($args))
+            $args = func_get_args();
+
+        assert(count($args) >= 1);
+        assert(is_string($args[0]));
+        $this->_tag = array_shift($args);
+        
+        if ($args && is_array($args[0]))
+            $this->_attr = array_shift($args);
+        else {
+            $this->_attr = array();
+            if ($args && $args[0] === false)
+                array_shift($args);
+        }
+        $this->setContent($args);
         $this->_properties = HTML::getTagProperties($this->_tag);
     }
 
@@ -76,7 +97,7 @@ function HTML (/* $content, ... */) {
 
 class HTML extends HtmlElement {
     function raw ($html_text) {
-        return new RawXML($html_text);
+        return new RawXml($html_text);
     }
     
     function getTagProperties($tag) {
@@ -89,10 +110,12 @@ class HTML extends HtmlElement {
         if (is_string($tags))
             $tags = preg_split('/\s+/', $tags);
         foreach ($tags as $tag) {
-            if (isset($props[$tag]))
-                $props[$tag] |= $prop_flag;
-            else
-                $props[$tag] = $prop_flag;
+            $tag = trim($tag);
+            if ($tag)
+                if (isset($props[$tag]))
+                    $props[$tag] |= $prop_flag;
+                else
+                    $props[$tag] = $prop_flag;
         }
     }
 
@@ -327,6 +350,10 @@ class HTML extends HtmlElement {
         $el = new HtmlElement('input');
         return $el->_init2(func_get_args());
     }
+    function button (/*...*/) {
+        $el = new HtmlElement('button');
+        return $el->_init2(func_get_args());
+    }
     function option (/*...*/) {
         $el = new HtmlElement('option');
         return $el->_init2(func_get_args());
@@ -339,6 +366,10 @@ class HTML extends HtmlElement {
         $el = new HtmlElement('textarea');
         return $el->_init2(func_get_args());
     }
+    function label (/*...*/) {
+        $el = new HtmlElement('label');
+        return $el->_init2(func_get_args());
+    }
 
     /****************************************/
     function area (/*...*/) {
@@ -365,6 +396,14 @@ class HTML extends HtmlElement {
         $el = new HtmlElement('nobody');
         return $el->_init2(func_get_args());
     }
+    function object (/*...*/) {
+        $el = new HtmlElement('object');
+        return $el->_init2(func_get_args());
+    }
+    function embed (/*...*/) {
+        $el = new HtmlElement('embed');
+        return $el->_init2(func_get_args());
+    }
 }
 
 define('HTMLTAG_EMPTY', 1);
@@ -379,8 +418,8 @@ HTML::_setTagProperty(HTMLTAG_ACCEPTS_INLINE,
                       'b big i small tt ' // %fontstyle
                       . 's strike u ' // (deprecated)
                       . 'abbr acronym cite code dfn em kbd samp strong var ' //%phrase
-                      . 'a img object br script map q sub sup span bdo '//%special
-                      . 'button input label option select textarea ' //%formctl
+                      . 'a img object embed br script map q sub sup span bdo '//%special
+                      . 'button input label option select textarea label ' //%formctl
 
                       // %block elements which contain inline content
                       . 'address h1 h2 h3 h4 h5 h6 p pre '
@@ -390,7 +429,7 @@ HTML::_setTagProperty(HTMLTAG_ACCEPTS_INLINE,
                       // other with inline content
                       . 'caption dt label legend '
                       // other with either inline or block
-                      . 'dd del ins li td th colgroup ');
+                      . 'dd del ins li td th colgroup');
 
 HTML::_setTagProperty(HTMLTAG_INLINE,
                       // %inline elements:
@@ -428,7 +467,7 @@ function HiddenInputs ($query_args, $pfx = false, $exclude = array()) {
     $inputs = HTML();
 
     foreach ($query_args as $key => $val) {
-        if (in_array($key,$exclude)) continue;
+        if (in_array($key, $exclude)) continue;
         $name = $pfx ? $pfx . "[$key]" : $key;
         if (is_array($val))
             $inputs->pushContent(HiddenInputs($val, $name));
@@ -444,12 +483,23 @@ function HiddenInputs ($query_args, $pfx = false, $exclude = array()) {
 /** Generate a <script> tag containing javascript.
  *
  * @param string $js  The javascript.
+ * @param string $script_args  (optional) hash of script tags options
+ *                             e.g. to provide another version or the defer attr
  * @return HtmlElement A <script> element.
  */
-function JavaScript($js) {
-    return HTML::script(array('language' => 'JavaScript',
-                              'type'     => 'text/javascript'),
-                        new RawXml("<!-- //\n${js}\n// -->"));
+function JavaScript ($js, $script_args = false) {
+    $default_script_args = array(//'version' => 'JavaScript', // not xhtml conformant
+                                 'type' => 'text/javascript');
+    $script_args = $script_args ? array_merge($default_script_args, $script_args)
+                                : $default_script_args;
+    if (empty($js))
+        return HTML::script($script_args);
+    else
+        // see http://devedge.netscape.com/viewsource/2003/xhtml-style-script/
+        return HTML::script($script_args,
+                            new RawXml((ENABLE_XHTML_XML ? "\n//<![CDATA[" : "\n<!--//")
+                                       . "\n".rtrim($js)."\n"
+                                       . (ENABLE_XHTML_XML ? "//]]>\n" : "// -->")));
 }
 
 /** Conditionally display content based of whether javascript is supported.
@@ -485,6 +535,81 @@ function IfJavaScript($if_content = false, $else_content = false) {
     
 /**
  $Log: HtmlElement.php,v $
+ Revision 1.46  2005/01/25 06:50:33  rurban
+ added label
+
+ Revision 1.45  2005/01/10 18:05:56  rurban
+ php5 case-sensitivity
+
+ Revision 1.44  2005/01/08 20:58:19  rurban
+ ending space after colgroup breaks _setTagProperty
+
+ Revision 1.43  2004/11/21 11:59:14  rurban
+ remove final \n to be ob_cache independent
+
+ Revision 1.42  2004/09/26 17:09:23  rurban
+ add SVG support for Ploticus (and hopefully all WikiPluginCached types)
+ SWF not yet.
+
+ Revision 1.41  2004/08/05 17:31:50  rurban
+ more xhtml conformance fixes
+
+ Revision 1.40  2004/06/25 14:29:17  rurban
+ WikiGroup refactoring:
+   global group attached to user, code for not_current user.
+   improved helpers for special groups (avoid double invocations)
+ new experimental config option ENABLE_XHTML_XML (fails with IE, and document.write())
+ fixed a XHTML validation error on userprefs.tmpl
+
+ Revision 1.39  2004/05/17 13:36:49  rurban
+ Apply RFE #952323 "ExternalSearchPlugin improvement", but
+   with <button><img></button>
+
+ Revision 1.38  2004/05/12 10:49:54  rurban
+ require_once fix for those libs which are loaded before FileFinder and
+   its automatic include_path fix, and where require_once doesn't grok
+   dirname(__FILE__) != './lib'
+ upgrade fix with PearDB
+ navbar.tmpl: remove spaces for IE &nbsp; button alignment
+
+ Revision 1.37  2004/04/26 20:44:34  rurban
+ locking table specific for better databases
+
+ Revision 1.36  2004/04/19 21:51:41  rurban
+ php5 compatibility: it works!
+
+ Revision 1.35  2004/04/19 18:27:45  rurban
+ Prevent from some PHP5 warnings (ref args, no :: object init)
+   php5 runs now through, just one wrong XmlElement object init missing
+ Removed unneccesary UpgradeUser lines
+ Changed WikiLink to omit version if current (RecentChanges)
+
+ Revision 1.34  2004/03/24 19:39:02  rurban
+ php5 workaround code (plus some interim debugging code in XmlElement)
+   php5 doesn't work yet with the current XmlElement class constructors,
+   WikiUserNew does work better than php4.
+ rewrote WikiUserNew user upgrading to ease php5 update
+ fixed pref handling in WikiUserNew
+ added Email Notification
+ added simple Email verification
+ removed emailVerify userpref subclass: just a email property
+ changed pref binary storage layout: numarray => hash of non default values
+ print optimize message only if really done.
+ forced new cookie policy: delete pref cookies, use only WIKI_ID as plain string.
+   prefs should be stored in db or homepage, besides the current session.
+
+ Revision 1.33  2004/03/18 22:32:33  rurban
+ work to make it php5 compatible
+
+ Revision 1.32  2004/02/15 21:34:37  rurban
+ PageList enhanced and improved.
+ fixed new WikiAdmin... plugins
+ editpage, Theme with exp. htmlarea framework
+   (htmlarea yet committed, this is really questionable)
+ WikiUser... code with better session handling for prefs
+ enhanced UserPreferences (again)
+ RecentChanges for show_deleted: how should pages be deleted then?
+
  Revision 1.31  2003/02/27 22:47:26  dairiki
  New functions in HtmlElement:
 
@@ -523,4 +648,4 @@ function IfJavaScript($if_content = false, $else_content = false) {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
+?>
\ No newline at end of file
