@@ -1,5 +1,7 @@
-<?php rcs_id('$Id: InlineParser.php,v 1.29 2003/09/13 22:43:00 carstenklapp Exp $');
-/* Copyright (C) 2002, Geoffrey T. Dairiki <dairiki@dairiki.org>
+<?php 
+rcs_id('$Id: InlineParser.php,v 1.69 2005/09/14 05:57:19 rurban Exp $');
+/* Copyright (C) 2002 Geoffrey T. Dairiki <dairiki@dairiki.org>
+ * Copyright (C) 2004,2005 Reini Urban
  *
  * This file is part of PhpWiki.
  * 
@@ -33,10 +35,9 @@
  */
 define('ESCAPE_CHAR', '~');
 
-require_once('lib/HtmlElement.php');
+require_once(dirname(__FILE__).'/HtmlElement.php');
 require_once('lib/CachedMarkup.php');
-require_once('lib/interwiki.php');
-require_once('lib/stdlib.php');
+require_once(dirname(__FILE__).'/stdlib.php');
 
 
 function WikiEscape($text) {
@@ -57,17 +58,14 @@ class RegexpSet_match {
      * The text leading up the the next match.
      */
     var $prematch;
-
     /**
      * The matched text.
      */
     var $match;
-
     /**
      * The text following the matched text.
      */
     var $postmatch;
-
     /**
      * Index of the regular expression which matched.
      */
@@ -89,7 +87,9 @@ class RegexpSet
      * look-ahead and look-behind assertions are okay.)
      */
     function RegexpSet ($regexps) {
-        $this->_regexps = $regexps;
+        assert($regexps);
+        $this->_regexps = array_unique($regexps);
+        if (!defined('_INLINE_OPTIMIZATION')) define('_INLINE_OPTIMIZATION',0);
     }
 
     /**
@@ -138,30 +138,86 @@ class RegexpSet
         $repeat = sprintf('{%d,}?', $pos + 1);
         return $this->_match($text, $this->_regexps, $repeat);
     }
-    
 
+    // Syntax: http://www.pcre.org/pcre.txt
+    //   x - EXTENDED, ignore whitespace
+    //   s - DOTALL
+    //   A - ANCHORED
+    //   S - STUDY
     function _match ($text, $regexps, $repeat) {
-        $pat= "/ ( . $repeat ) ( (" . join(')|(', $regexps) . ") ) /Axs";
-
-        if (! preg_match($pat, $text, $m)) {
-            return false;
+        // If one of the regexps is an empty string, php will crash here: 
+        // sf.net: Fatal error: Allowed memory size of 8388608 bytes exhausted 
+        //         (tried to allocate 634 bytes)
+        if (_INLINE_OPTIMIZATION) { // disabled, wrong
+        // So we try to minize memory usage, by looping explicitly,
+        // and storing only those regexp which actually match. 
+        // There may be more than one, so we have to find the longest, 
+        // and match inside until the shortest is empty.
+	$matched = array(); $matched_ind = array();
+        for ($i=0; $i<count($regexps); $i++) {
+            if (!trim($regexps[$i])) {
+                trigger_error("empty regexp $i", E_USER_WARNING);
+                continue;
+            }
+            $pat= "/ ( . $repeat ) ( " . $regexps[$i] . " ) /x";
+            if (preg_match($pat, $text, $_m)) {
+            	$m = $_m; // FIXME: prematch, postmatch is wrong
+                $matched[] = $regexps[$i];
+                $matched_ind[] = $i;
+                $regexp_ind = $i;
+            }
+        }
+        // To overcome ANCHORED:
+        // We could sort by longest match and iterate over these.
+        if (empty($matched)) return false;
+        }
+        $match = new RegexpSet_match;
+        
+        // Optimization: if the matches are only "$" and another, then omit "$"
+        if (! _INLINE_OPTIMIZATION or count($matched) > 2) {
+            assert(!empty($repeat));
+            assert(!empty($regexps));
+            for ($i=0; $i<count($regexps); $i++) {
+                if (!trim($regexps[$i])) {
+                    trigger_error("empty regexp $i", E_USER_WARNING);
+                    $regexps[$i] = '\Wxxxx\w\W\w\W\w\W\w\W\w\W\w'; // some placeholder
+                }
+            }
+            // We could do much better, if we would know the matching markup for the 
+            // longest regexp match:
+            $hugepat= "/ ( . $repeat ) ( (" . join(')|(', $regexps) . ") ) /Asx";
+            // Proposed premature optimization 1:
+            //$hugepat= "/ ( . $repeat ) ( (" . join(')|(', array_values($matched)) . ") ) /Asx";
+            if (! preg_match($hugepat, $text, $m)) {
+                return false;
+            }
+            // Proposed premature optimization 1:
+            //$match->regexp_ind = $matched_ind[count($m) - 4];
+            $match->regexp_ind = count($m) - 4;
+        } else {
+            $match->regexp_ind = $regexp_ind;
         }
         
-        $match = new RegexpSet_match;
         $match->postmatch = substr($text, strlen($m[0]));
         $match->prematch = $m[1];
         $match->match = $m[2];
-        $match->regexp_ind = count($m) - 4;
 
-        /* DEBUGGING
+        /* DEBUGGING */
+        /*
+        if (DEBUG & 4) {
+          var_dump($regexps); var_dump($matched); var_dump($matched_inc); 
         PrintXML(HTML::dl(HTML::dt("input"),
                           HTML::dd(HTML::pre($text)),
+                          HTML::dt("regexp"),
+                          HTML::dd(HTML::pre($match->regexp_ind, ":", $regexps[$match->regexp_ind])),
+                          HTML::dt("prematch"),
+                          HTML::dd(HTML::pre($match->prematch)),
                           HTML::dt("match"),
                           HTML::dd(HTML::pre($match->match)),
-                          HTML::dt("regexp"),
-                          HTML::dd(HTML::pre($regexps[$match->regexp_ind])),
-                          HTML::dt("prematch"),
-                          HTML::dd(HTML::pre($match->prematch))));
+                          HTML::dt("postmatch"),
+                          HTML::dd(HTML::pre($match->postmatch))
+                          ));
+        }
         */
         return $match;
     }
@@ -204,7 +260,7 @@ class SimpleMarkup
 /**
  * A balanced markup rule.
  *
- * These are defined by a start regexp, and and end regexp.
+ * These are defined by a start regexp, and an end regexp.
  */ 
 class BalancedMarkup
 {
@@ -254,30 +310,64 @@ class Markup_escape  extends SimpleMarkup
     }
 }
 
+/**
+ * [image.jpg size=50% border=5], [image.jpg size=50x30]
+ * Support for the following attributes: see stdlib.php:LinkImage()
+ *   size=<precent>%, size=<width>x<height>
+ *   border=n, align=\w+, hspace=n, vspace=n
+ */
+function isImageLink($link) {
+    if (!$link) return false;
+    assert(defined('INLINE_IMAGES'));
+    return preg_match("/\\.(" . INLINE_IMAGES . ")$/i", $link)
+        or preg_match("/\\.(" . INLINE_IMAGES . ")\s+(size|border|align|hspace|vspace)=/i", $link);
+}
+
 function LinkBracketLink($bracketlink) {
-    global $request, $AllowedProtocols, $InlineImages;
 
-    include_once("lib/interwiki.php");
-    $intermap = InterWikiMap::GetMap($request);
-    
     // $bracketlink will start and end with brackets; in between will
     // be either a page name, a URL or both separated by a pipe.
     
     // strip brackets and leading space
+    // FIXME: \n inside [] will lead to errors
     preg_match('/(\#?) \[\s* (?: (.*?) \s* (?<!' . ESCAPE_CHAR . ')(\|) )? \s* (.+?) \s*\]/x',
 	       $bracketlink, $matches);
+    if (count($matches) < 4) {
+    	trigger_error(_("Invalid [] syntax ignored").": ".$bracketlink, E_USER_WARNING);
+    	return new Cached_Link;
+    }
     list (, $hash, $label, $bar, $rawlink) = $matches;
 
     $label = UnWikiEscape($label);
-    $link = UnWikiEscape($rawlink);
+    /*
+     * Check if the user has typed a explicit URL. This solves the
+     * problem where the URLs have a ~ character, which would be stripped away.
+     *   "[http:/server/~name/]" will work as expected
+     *   "http:/server/~name/"   will NOT work as expected, will remove the ~
+     */
+    if (strstr($rawlink, "http://") or strstr($rawlink, "https://")) {
+        $link = $rawlink;
+        // Mozilla Browser URI Obfuscation Weakness 2004-06-14
+        //   http://www.securityfocus.com/bid/10532/
+        //   goodurl+"%2F%20%20%20."+badurl
+        if (preg_match("/%2F(%20)+\./i", $rawlink)) {
+            $rawlink = preg_replace("/%2F(%20)+\./i","%2F.",$rawlink);
+        }
+    } else
+        $link  = UnWikiEscape($rawlink);
 
+    // [label|link]
     // if label looks like a url to an image, we want an image link.
-    if (preg_match("/\\.($InlineImages)$/i", $label)) {
+    if (isImageLink($label)) {
         $imgurl = $label;
-        if (! preg_match("#^($AllowedProtocols):#", $imgurl)) {
-            // linkname like 'images/next.gif'.
-            global $Theme;
-            $imgurl = $Theme->getImageURL($linkname);
+        $intermap = getInterwikiMap();
+        if (preg_match("/^" . $intermap->getRegexp() . ":/", $label)) {
+            $imgurl = $intermap->link($label);
+            $imgurl = $imgurl->getAttr('href');
+        } elseif (! preg_match("#^(" . ALLOWED_PROTOCOLS . "):#", $imgurl)) {
+            // local theme linkname like 'images/next.gif'.
+            global $WikiTheme;
+            $imgurl = $WikiTheme->getImageURL($imgurl);
         }
         $label = LinkImage($imgurl, $link);
     }
@@ -289,19 +379,37 @@ function LinkBracketLink($bracketlink) {
                        $bar ? $label : $link);
     }
 
-    if (preg_match("#^($AllowedProtocols):#", $link)) {
+    if (preg_match("#^(" . ALLOWED_PROTOCOLS . "):#", $link)) {
         // if it's an image, embed it; otherwise, it's a regular link
-        if (preg_match("/\\.($InlineImages)$/i", $link))
-            // no image link, just the src. see [img|link] above
+        if (isImageLink($link))
             return LinkImage($link, $label);
         else
             return new Cached_ExternalLink($link, $label);
     }
     elseif (preg_match("/^phpwiki:/", $link))
         return new Cached_PhpwikiURL($link, $label);
-    elseif (preg_match("/^" . $intermap->getRegexp() . ":/", $link))
+    /*
+     * Inline images in Interwiki urls's:
+     * [File:my_image.gif] inlines the image,
+     * File:my_image.gif shows a plain inter-wiki link,
+     * [what a pic|File:my_image.gif] shows a named inter-wiki link to the gif
+     * [File:my_image.gif|what a pic] shows a inlimed image linked to the page "what a pic"
+     *
+     * Note that for simplicity we will accept embedded object tags (non-images) 
+     * here also, and seperate them later in LinkImage()
+     */
+    elseif (strstr($link,':')
+            and ($intermap = getInterwikiMap()) 
+            and preg_match("/^" . $intermap->getRegexp() . ":/", $link)) 
+    {
+        // trigger_error("label: $label link: $link", E_USER_WARNING);
+        if (empty($label) and isImageLink($link)) {
+            // if without label => inlined image [File:xx.gif]
+            $imgurl = $intermap->link($link);
+            return LinkImage($imgurl->getAttr('href'), $label);
+        }
         return new Cached_InterwikiLink($link, $label);
-    else {
+    } else {
         // Split anchor off end of pagename.
         if (preg_match('/\A(.*)(?<!'.ESCAPE_CHAR.')#(.*?)\Z/', $rawlink, $m)) {
             list(,$rawlink,$anchor) = $m;
@@ -332,8 +440,7 @@ class Markup_bracketlink  extends SimpleMarkup
 class Markup_url extends SimpleMarkup
 {
     function getMatchRegexp () {
-        global $AllowedProtocols;
-        return "(?<![[:alnum:]]) (?:$AllowedProtocols) : [^\s<>\"']+ (?<![ ,.?; \] \) ])";
+        return "(?<![[:alnum:]]) (?:" . ALLOWED_PROTOCOLS . ") : [^\s<>\"']+ (?<![ ,.?; \] \) ])";
     }
     
     function markup ($match) {
@@ -346,13 +453,12 @@ class Markup_interwiki extends SimpleMarkup
 {
     function getMatchRegexp () {
         global $request;
-        $map = InterWikiMap::GetMap($request);
+        $map = getInterwikiMap();
         return "(?<! [[:alnum:]])" . $map->getRegexp(). ": \S+ (?<![ ,.?;! \] \) \" \' ])";
     }
 
     function markup ($match) {
-        global $request;
-        $map = InterWikiMap::GetMap($request);
+        //$map = getInterwikiMap();
         return new Cached_InterwikiLink(UnWikiEscape($match));
     }
 }
@@ -361,12 +467,14 @@ class Markup_wikiword extends SimpleMarkup
 {
     function getMatchRegexp () {
         global $WikiNameRegexp;
+        if (!trim($WikiNameRegexp)) return " " . WIKI_NAME_REGEXP;
         return " $WikiNameRegexp";
     }
 
     function markup ($match) {
+        if (!$match) return false;
         if ($this->_isWikiUserPage($match))
-            return $this->_UserLink($match);
+            return new Cached_UserLink($match); //$this->_UserLink($match);
         else
             return new Cached_WikiLink($match);
     }
@@ -376,7 +484,7 @@ class Markup_wikiword extends SimpleMarkup
         global $request;
         $dbi = $request->getDbh();
         $page_handle = $dbi->getPage($page);
-        if ($page_handle->get('pref'))
+        if ($page_handle and $page_handle->get('pref'))
             return true;
         else
             return false;
@@ -392,9 +500,10 @@ class Markup_wikiword extends SimpleMarkup
 
 class Markup_linebreak extends SimpleMarkup
 {
+    //var $_match_regexp = "(?: (?<! %) %%% (?! %) | <(?:br|BR)> | <(?:br|BR) \/> )";
     var $_match_regexp = "(?: (?<! %) %%% (?! %) | <(?:br|BR)> )";
 
-    function markup () {
+    function markup ($match) {
         return HTML::br();
     }
 }
@@ -462,17 +571,15 @@ class Markup_nestled_emphasis extends BalancedMarkup
         switch ($match) {
         case '*': return new HtmlElement('b', $body);
         case '=': return new HtmlElement('tt', $body);
-        case '_':  return new HtmlElement('i', $body);
+        case '_': return new HtmlElement('i', $body);
         }
     }
 }
 
 class Markup_html_emphasis extends BalancedMarkup
 {
-    var $_start_regexp = "<(?: b|big|i|small|tt|
-                               em|strong|
-                               abbr|acronym|cite|code|dfn|kbd|samp|var|
-                               sup|sub )>";
+    var $_start_regexp = 
+        "<(?: b|big|i|small|tt|em|strong|cite|code|dfn|kbd|samp|var|sup|sub )>";
 
     function getEndRegexp ($match) {
         return "<\\/" . substr($match, 1);
@@ -484,11 +591,135 @@ class Markup_html_emphasis extends BalancedMarkup
     }
 }
 
+class Markup_html_abbr extends BalancedMarkup
+{
+    //rurban: abbr|acronym need an optional title tag.
+    //sf.net bug #728595
+    var $_start_regexp = "<(?: abbr|acronym )(?: \stitle=[^>]*)?>";
+
+    function getEndRegexp ($match) {
+    	if (substr($match,1,4) == 'abbr')
+    	    $tag = 'abbr';
+    	else
+    	    $tag = 'acronym';
+        return "<\\/" . $tag . '>';
+    }
+    
+    function markup ($match, $body) {
+    	if (substr($match,1,4) == 'abbr')
+    	    $tag = 'abbr';
+    	else
+    	    $tag = 'acronym';
+    	$rest = substr($match,1+strlen($tag),-1);
+    	if (!empty($rest)) {
+    	    list($key,$val) = explode("=",$rest);
+    	    $args = array($key => $val);
+    	} else $args = array();
+        return new HtmlElement($tag, $args, $body);
+    }
+}
+
+/** ENABLE_MARKUP_COLOR
+ *  See http://www.pmwiki.org/wiki/PmWiki/WikiStyles and
+ *      http://www.flexwiki.com/default.aspx/FlexWiki/FormattingRules.html
+ */
+class Markup_color extends BalancedMarkup {
+    // %color=blue% blue text %% and back to normal
+    var $_start_regexp = "%color=(?: [^%]*)%";
+    var $_end_regexp = "%%";
+    
+    function markup ($match, $body) {
+    	$color = substr($match, 7, -1);
+        if (strlen($color) != 7 
+            and in_array($color, array('red', 'blue', 'grey', 'black'))) {
+            // must be a name
+            return new HtmlElement('font', array('color' => $color), $body);
+        } elseif ((substr($color,0,1) == '#') 
+                  and (strspn(substr($color,1),'0123456789ABCDEFabcdef') == strlen($color)-1)) {
+            return new HtmlElement('font', array('color' => $color), $body);
+        } else {
+            trigger_error(sprintf(_("unknown color %s ignored"), $color), E_USER_WARNING);
+        }
+        	
+    }
+}
+
+// Special version for single-line plugins formatting, 
+//  like: '<small>< ?plugin PopularNearby ? ></small>'
+class Markup_plugin extends SimpleMarkup
+{
+    var $_match_regexp = '<\?plugin(?:-form)?\s[^\n]+?\?>';
+
+    function markup ($match) {
+	//$xml = new Cached_PluginInvocation($match);
+	//$xml->setTightness(true,true);
+	return new Cached_PluginInvocation($match);
+    }
+}
+
+/** ENABLE_MARKUP_TEMPLATE
+ *  Template syntax similar to mediawiki
+ *  {{template}}
+ * => <?plugin Template page=template?>
+ *  {{template|var=value|...}}
+ * => <?plugin Template page=template vars="var=value&..."?>
+ */
+class Markup_template_plugin  extends SimpleMarkup
+{
+    var $_match_regexp = '\{\{\w[^\n]+\}\}';
+    
+    function markup ($match) {
+        $page = substr($match,2,-2); $vars = '';
+        if (preg_match('/^(\S+)\|(.*)$/', $page, $_m)) {
+            $page = $_m[1];
+            $vars = str_replace('|', '&', $_m[2]);
+        }
+        if ($vars)
+    	    $s = '<?plugin Template page=' . $page . ' vars="' . $vars . '"?>';
+    	else
+    	    $s = '<?plugin Template page=' . $page . '?>';
+	return new Cached_PluginInvocation($s);
+    }
+}
+
+// TODO: "..." => "&#133;"  browser specific display (not cached?)
+// TODO: "--" => "&emdash;" browser specific display (not cached?)
+// TODO: Support more HTML::Entities: (C) for copy, --- for mdash, -- for ndash
+
+class Markup_html_entities  extends SimpleMarkup {
+    var $_match_regexp = '(: \.\.\.|\-\-|\-\-\-|\(C\) )';
+   
+    function markup ($match) {
+        static $entities = array('...'  => '&#133;',
+                                 '--'   => '&ndash;',
+                                 '---'  => '&mdash;',
+                                 '(C)'  => '&copy;',
+                                 );
+        return HTML::Raw($entities[$match]);
+    }
+}
+
+class Markup_isonumchars  extends SimpleMarkup {
+    var $_match_regexp = '\&\#\d{2,5};';
+    
+    function markup ($match) {
+        return HTML::Raw($match);
+    }
+}
+
+class Markup_isohexchars extends SimpleMarkup {
+    // hexnums, like &#x00A4; <=> &curren;
+    var $_match_regexp = '\&\#x[0-9a-fA-F]{2,4};';
+    
+    function markup ($match) {
+        return HTML::Raw($match);
+    }
+}
+
 // FIXME: Do away with magic phpwiki forms.  (Maybe phpwiki: links too?)
 // FIXME: Do away with plugin-links.  They seem not to be used.
 //Plugin link
 
-
 class InlineTransformer
 {
     var $_regexps = array();
@@ -499,12 +730,17 @@ class InlineTransformer
             $markup_types = array('escape', 'bracketlink', 'url',
                                   'interwiki', 'wikiword', 'linebreak',
                                   'old_emphasis', 'nestled_emphasis',
-                                  'html_emphasis');
-
+                                  'html_emphasis', 'html_abbr', 'plugin',
+                                  'isonumchars', 'isohexchars', /*'html_entities',*/
+                                  );
         foreach ($markup_types as $mtype) {
             $class = "Markup_$mtype";
             $this->_addMarkup(new $class);
         }
+        if (ENABLE_MARKUP_COLOR)
+            $this->_addMarkup(new Markup_color);
+        if (defined("ENABLE_MARKUP_TEMPLATE") and ENABLE_MARKUP_TEMPLATE)
+            $this->_addMarkup(new Markup_template_plugin);
     }
 
     function _addMarkup ($markup) {
@@ -523,6 +759,7 @@ class InlineTransformer
 
         // $end_re takes precedence: "favor reduce over shift"
         array_unshift($regexps, $end_regexps[0]);
+        //array_push($regexps, $end_regexps[0]);
         $regexps = new RegexpSet($regexps);
         
         $input = $text;
@@ -534,6 +771,10 @@ class InlineTransformer
             if ($match->regexp_ind == 0) {
                 // No start pattern found before end pattern.
                 // We're all done!
+                if (isset($markup) and is_object($markup) and isa($markup,'Markup_plugin')) {
+                    $current =& $output->_content[count($output->_content)-1];
+                    $current->setTightness(true,true);
+                }
                 $output->pushContent($match->prematch);
                 $text = $match->postmatch;
                 return $output;
@@ -550,9 +791,15 @@ class InlineTransformer
 
             // Matched markup.  Eat input, push output.
             // FIXME: combine adjacent strings.
+            if (isa($markup, 'SimpleMarkup'))
+                $current = $markup->markup($match->match);
+            else
+                $current = $markup->markup($match->match, $body);
             $input = $match->postmatch;
-            $output->pushContent($match->prematch,
-                                 $markup->markup($match->match, $body));
+            if (isset($markup) and is_object($markup) and isa($markup,'Markup_plugin')) {
+                $current->setTightness(true,true);
+            }
+            $output->pushContent($match->prematch, $current);
 
             $match = $regexps->match($input);
         }
@@ -566,7 +813,9 @@ class InlineTransformer
         if (isa($markup, 'SimpleMarkup'))
             return true;        // Done. SimpleMarkup is simple.
 
+        if (!is_object($markup)) return false; // Some error: Should assert
         array_unshift($end_regexps, $markup->getEndRegexp($match));
+
         // Optimization: if no end pattern in text, we know the
         // parse will fail.  This is an important optimization,
         // e.g. when text is "*lots *of *start *delims *with
@@ -620,6 +869,23 @@ function TransformLinks($text, $markup = 2.0, $basepage = false) {
     return $trfm->parse($text);
 }
 
+// $Log: InlineParser.php,v $
+// Revision 1.69  2005/09/14 05:57:19  rurban
+// make ENABLE_MARKUP_TEMPLATE optional
+//
+// Revision 1.68  2005/09/10 21:24:32  rurban
+// optionally support {{Template|vars}} syntax
+//
+// Revision 1.67  2005/06/06 17:41:20  rurban
+// support new ENABLE_MARKUP_COLOR
+//
+// Revision 1.66  2005/04/23 11:15:49  rurban
+// handle allowed inlined objects within INLINE_IMAGES
+//
+// Revision 1.65  2005/03/27 18:24:17  rurban
+// add Log
+//
+
 // (c-file-style: "gnu")
 // Local Variables:
 // mode: php
@@ -628,4 +894,4 @@ function TransformLinks($text, $markup = 2.0, $basepage = false) {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
+?>
\ No newline at end of file
