@@ -1,15 +1,24 @@
-<?php rcs_id('$Id: TextSearchQuery.php,v 1.5 2002/11/19 19:08:14 dairiki Exp $');
+<?php rcs_id('$Id: TextSearchQuery.php,v 1.21 2005/09/14 05:59:48 rurban Exp $');
 /**
- * A text search query.
+ * A text search query, converting queries to PCRE and SQL matchers.
  *
- * This represents "Google-like" text search queries like:
+ * This represents an enhanced "Google-like" text search query:
  * <dl>
+ * <dt> default: case-insensitive glob-style search with special operators OR AND NOT -
  * <dt> wiki -test
- *   <dd> Match strings containing the substring 'wiki',  and not containing the
+ *   <dd> Match strings containing the substring 'wiki', and NOT containing the
  *        substring 'test'.
  * <dt> wiki word or page
- *   <dd> Match strings containing the substring 'wiki' and either the substring
- *        'word' or the substring 'page'.
+ *   <dd> Match strings containing the substring 'wiki' AND either the substring
+ *        'word' OR the substring 'page'.
+ * <dt> auto-detect regex hints, glob-style or regex-style, and converts them 
+ *      to PCRE and SQL matchers:
+ *   <dd> "^word$" => EXACT(word)
+ *   <dd> "^word"  => STARTS_WITH(word)
+ *   <dd> "word*"  => STARTS_WITH(word)
+ *   <dd> "*word"  => ENDS_WITH(word)
+ *   <dd> "/^word.* /" => REGEX(^word.*)
+ *   <dd> "word*word" => REGEX(word.*word)
  * </dl>
  *
  * The full query syntax, in order of precedence, is roughly:
@@ -18,7 +27,7 @@
  * following search clause.
  *
  * Search clauses may be joined with the (left-associative) binary operators
- * 'AND' and 'OR'.
+ * 'AND' and 'OR'. (case-insensitive)
  *
  * Two adjoining search clauses are joined with an implicit 'AND'.  This has
  * lower precedence than either an explicit 'AND' or 'OR', so "a b OR c"
@@ -32,19 +41,49 @@
  * quotes.  If you wan't to include the quote character within a quoted string,
  * double-up on the quote character: 'I''m hungry' is equivalent to
  * "I'm hungry".
+ *
+ * Force regex on "re:word" => posix-style, "/word/" => pcre-style 
+ * or use regex='glob' to use file wildcard-like matching. (not yet)
+ *
+ * The parsed tree is then converted to the needed PCRE (highlight, 
+ * simple backends) or SQL functions. 
+ *
+ * @author: Jeff Dairiki
+ * @author: Reini Urban (case and regex detection, enhanced sql callbacks)
  */
+
+// regex-style: 'auto', 'none', 'glob', 'posix', 'pcre', 'sql'
+define ('TSQ_REGEX_NONE', 0);
+define ('TSQ_REGEX_AUTO', 1);
+define ('TSQ_REGEX_POSIX', 2);
+define ('TSQ_REGEX_GLOB', 4);
+define ('TSQ_REGEX_PCRE', 8);
+define ('TSQ_REGEX_SQL', 16);
+
 class TextSearchQuery {
     /**
      * Create a new query.
      *
      * @param $search_query string The query.  Syntax is as described above.
      * Note that an empty $search_query will match anything.
+     * @param $case_exact boolean
+     * @param $regex string one of 'auto', 'none', 'glob', 'posix', 'pcre', 'sql'
      * @see TextSearchQuery
      */
-    function TextSearchQuery($search_query) {
+    function TextSearchQuery($search_query, $case_exact=false, $regex='auto') {
+        if ($regex == 'none' or !$regex) 
+            $this->_regex = 0;
+        elseif (defined("TSQ_REGEX_".strtoupper($regex)))
+            $this->_regex = constant("TSQ_REGEX_".strtoupper($regex));
+        else {
+            trigger_error(fmt("Unsupported argument: %s=%s", 'regex', $regex));
+            $this->_regex = 0;
+        }
+        $this->_case_exact = $case_exact;
         $parser = new TextSearchQuery_Parser;
-        $this->_tree = $parser->parse($search_query);
-        $this->_optimize();
+        $this->_tree = $parser->parse($search_query, $case_exact, $this->_regex);
+        $this->_optimize(); // broken under certain circumstances: "word -word -word"
+        $this->_stoplist = '(A|An|And|But|By|For|From|In|Is|It|Of|On|Or|The|To|With)';
     }
 
     function _optimize() {
@@ -52,11 +91,15 @@ class TextSearchQuery {
     }
 
     /**
-     * Get a regexp which matches the query.
+     * Get a PCRE regexp which matches the query.
      */
     function asRegexp() {
-        if (!isset($this->_regexp))
-            $this->_regexp =  '/^' . $this->_tree->regexp() . '/isS';
+        if (!isset($this->_regexp)) {
+            if ($this->_regex)
+                $this->_regexp =  '/' . $this->_tree->regexp() . '/'.($this->_case_exact?'':'i').'sS';
+            else
+                $this->_regexp =  '/^' . $this->_tree->regexp() . '/'.($this->_case_exact?'':'i').'sS';
+        }
         return $this->_regexp;
     }
 
@@ -84,8 +127,7 @@ class TextSearchQuery {
             $words = array_unique($this->_tree->highlight_words());
             if (!$words) {
                 $this->_hilight_regexp = false;
-            }
-            else {
+            } else {
                 foreach ($words as $key => $word)
                     $words[$key] = preg_quote($word, '/');
                 $this->_hilight_regexp = '(?:' . join('|', $words) . ')';
@@ -95,7 +137,7 @@ class TextSearchQuery {
     }
 
     /**
-     * Make an SQL clause which matches the query.
+     * Make an SQL clause which matches the query. (deprecated, use makeSqlClause instead)
      *
      * @param $make_sql_clause_cb WikiCallback
      * A callback which takes a single word as an argument and
@@ -103,7 +145,11 @@ class TextSearchQuery {
      * containing the word.  The word passed to the callback will always
      * be in all lower case.
      *
-     * Example usage:
+     * TODO: support db-specific extensions, like MATCH AGAINST or REGEX
+     *       mysql => 4.0.1 can also do Google: MATCH AGAINST IN BOOLEAN MODE
+     *       How? WikiDB backend method?
+     *
+     * Old example usage:
      * <pre>
      *     function sql_title_match($word) {
      *         return sprintf("LOWER(title) like '%s'",
@@ -119,16 +165,16 @@ class TextSearchQuery {
      * This will result in $sql_clause containing something like
      * "(LOWER(title) like 'wiki') AND NOT (LOWER(title) like 'page')".
      *
-     * @return string The PCRE regexp.
+     * @return string The SQL clause.
      */
-    function makeSqlClause($make_sql_clause_cb) {
-        $this->_sql_clause_cb = $make_sql_clause_cb;
+    function makeSqlClause($sql_clause_cb) {
+        $this->_sql_clause_cb = $sql_clause_cb;
         return $this->_sql_clause($this->_tree);
     }
-
+    // deprecated: use _sql_clause_obj now.
     function _sql_clause($node) {
         switch ($node->op) {
-        case 'WORD':
+        case 'WORD':        // word => %word%
             return $this->_sql_clause_cb->call($node->word);
         case 'NOT':
             return "NOT (" . $this->_sql_clause($node->leaves[0]) . ")";
@@ -139,11 +185,40 @@ class TextSearchQuery {
                 $subclauses[] = "(" . $this->_sql_clause($leaf) . ")";
             return join(" $node->op ", $subclauses);
         default:
-            assert($node->op == 'VOID');
+            assert($node->op == VOID);
             return '1=1';
         }
     }
 
+    /** Get away with the callback and use a db-specific search class instead.
+     * @see WikiDB_backend_PearDB_search
+     */
+    function makeSqlClauseObj(&$sql_search_cb) {
+        $this->_sql_clause_cb = $sql_search_cb;
+        return $this->_sql_clause_obj($this->_tree);
+    }
+
+    function _sql_clause_obj($node) {
+        switch ($node->op) {
+        case 'NOT':
+            return "NOT (" . $this->_sql_clause_cb->call($node->leaves[0]) . ")";
+        case 'AND':
+        case 'OR':
+            $subclauses = array();
+            foreach ($node->leaves as $leaf)
+                $subclauses[] = "(" . $this->_sql_clause_obj($leaf) . ")";
+            return join(" $node->op ", $subclauses);
+        case 'VOID':
+            return '0=1';
+        case 'ALL':
+            return '1=1';
+        default:
+            return $this->_sql_clause_cb->call($node);
+        }
+    }
+
+    function sql() { return '%'.$this->_sql_quote($this->word).'%'; }
+
     /**
      * Get printable representation of the parse tree.
      *
@@ -160,6 +235,8 @@ class TextSearchQuery {
             return $indent . "WORD: $node->word";
         case 'VOID':
             return $indent . "VOID";
+        case 'ALL':
+            return $indent . "ALL";
         default:
             $lines = array($indent . $node->op . ":");
             $indent .= "  ";
@@ -196,7 +273,7 @@ class NullTextSearchQuery extends TextSearchQuery {
 /**
  * Virtual base class for nodes in a TextSearchQuery parse tree.
  *
- * Also servers as a 'VOID' (contentless) node.
+ * Also serves as a 'VOID' (contentless) node.
  */
 class TextSearchQuery_node
 {
@@ -224,6 +301,8 @@ class TextSearchQuery_node
     function highlight_words($negated = false) {
         return array();
     }
+
+    function sql()    { return $this->word; }
 }
 
 /**
@@ -237,16 +316,71 @@ extends TextSearchQuery_node
     function TextSearchQuery_node_word($word) {
         $this->word = $word;
     }
-
     function regexp() {
         return '(?=.*' . preg_quote($this->word, '/') . ')';
     }
-
     function highlight_words($negated = false) {
         return $negated ? array() : array($this->word);
     }
+    function _sql_quote() {
+        $word = preg_replace('/(?=[%_\\\\])/', "\\", $this->word);
+        return $GLOBALS['request']->_dbi->qstr($word);
+    }
+    function sql()    { return '%'.$this->_sql_quote($this->word).'%'; }
 }
 
+class TextSearchQuery_node_all
+extends TextSearchQuery_node {
+    var $op = "ALL";
+    function regexp() { return '(?=.*)'; }
+    function sql()    { return '%'; }
+}
+class TextSearchQuery_node_starts_with
+extends TextSearchQuery_node_word {
+    var $op = "STARTS_WITH";
+    function regexp() { return '(?=.*\b' . preg_quote($this->word, '/') . ')'; }
+    function sql()    { return $this->_sql_quote($this->word).'%'; }
+}
+
+class TextSearchQuery_node_ends_with
+extends TextSearchQuery_node_word {
+    var $op = "ENDS_WITH";
+    function regexp() { return '(?=.*' . preg_quote($this->word, '/') . '\b)'; }
+    function sql()    { return '%'.$this->_sql_quote($this->word); }
+}
+
+class TextSearchQuery_node_exact
+extends TextSearchQuery_node_word {
+    var $op = "EXACT";
+    function regexp() { return '(?=\b' . preg_quote($this->word, '/') . '\b)'; }
+    function sql()    { return $this->_sql_squote($this->word); }
+}
+
+class TextSearchQuery_node_regex // posix regex. FIXME!
+extends TextSearchQuery_node_word {
+    var $op = "REGEX"; // using REGEXP or ~ extension
+    function regexp() { return '(?=.*\b' . $this->word . '\b)'; }
+    function sql()    { return $this->_sql_quote($this->word); }
+}
+
+class TextSearchQuery_node_regex_glob
+extends TextSearchQuery_node_regex {
+    var $op = "REGEX_GLOB";
+    function regexp() { return '(?=.*\b' . glob_to_pcre($this->word) . '\b)'; }
+}
+
+class TextSearchQuery_node_regex_pcre // how to handle pcre modifiers? /i
+extends TextSearchQuery_node_regex {
+    var $op = "REGEX_PCRE";
+    function regexp() { return $this->word; }
+}
+
+class TextSearchQuery_node_regex_sql
+extends TextSearchQuery_node_regex {
+    var $op = "REGEX_SQL"; // using LIKE
+    function regexp() { return str_replace(array("/%/","/_/"), array(".*","."), $this->word); }
+    function sql()    { return $this->word; }
+}
 
 /**
  * A negated clause.
@@ -356,6 +490,11 @@ extends TextSearchQuery_node_binop
         return $this;
     }
 
+    /* FIXME!
+     * Either we need all combinations of all words to be position independent,
+     * or we have to use multiple match calls for each AND
+     * (AND x y) => /(?(:x)(:y))|(?(:y)(:x))/
+     */
     function regexp() {
         $regexp = '';
         foreach ($this->leaves as $leaf)
@@ -407,13 +546,24 @@ extends TextSearchQuery_node_binop
 ////////////////////////////////////////////////////////////////
 //
 // Parser:
+//   op's (and, or, not) are forced to lowercase in the tokenizer.
 //
 ////////////////////////////////////////////////////////////////
-define ('TSQ_TOK_WORD',   1);
-define ('TSQ_TOK_BINOP',  2);
-define ('TSQ_TOK_NOT',    4);
-define ('TSQ_TOK_LPAREN', 8);
-define ('TSQ_TOK_RPAREN', 16);
+define ('TSQ_TOK_BINOP',  1);
+define ('TSQ_TOK_NOT',    2);
+define ('TSQ_TOK_LPAREN', 4);
+define ('TSQ_TOK_RPAREN', 8);
+define ('TSQ_TOK_WORD',   16);
+define ('TSQ_TOK_STARTS_WITH', 32);
+define ('TSQ_TOK_ENDS_WITH', 64);
+define ('TSQ_TOK_EXACT', 128);
+define ('TSQ_TOK_REGEX', 256);
+define ('TSQ_TOK_REGEX_GLOB', 512);
+define ('TSQ_TOK_REGEX_PCRE', 1024);
+define ('TSQ_TOK_REGEX_SQL', 2048);
+define ('TSQ_TOK_ALL', 4096);
+// all bits from word to the last.
+define ('TSQ_ALLWORDS', (4096*2)-1 - (16-1));
 
 class TextSearchQuery_Parser 
 {
@@ -441,17 +591,24 @@ class TextSearchQuery_Parser
      * The terminal tokens are:
      *
      *
-     * and|or		BINOP
-     * -|not		NOT
-     * (		LPAREN
-     * )		RPAREN
-     * [^-()\s][^()\s]*	WORD
-     * "[^"]*"		WORD
-     * '[^']*'		WORD
+     * and|or		  BINOP
+     * -|not		  NOT
+     * (		  LPAREN
+     * )		  RPAREN
+     * /[^-()\s][^()\s]*  WORD
+     * /"[^"]*"/	  WORD
+     * /'[^']*'/	  WORD
+     *
+     * ^WORD              STARTS_WITH
+     * WORD*              STARTS_WITH
+     * *WORD              ENDS_WITH
+     * ^WORD$             EXACT
+     * *                  ALL
      */
 
-    function parse ($search_expr) {
-        $this->lexer = new TextSearchQuery_Lexer($search_expr);
+    function parse ($search_expr, $case_exact=false, $regex=TSQ_REGEX_AUTO) {
+        $this->lexer = new TextSearchQuery_Lexer($search_expr, $case_exact, $regex);
+        $this->_regex = $regex;
         $tree = $this->get_list('toplevel');
         assert($this->lexer->eof());
         unset($this->lexer);
@@ -469,7 +626,6 @@ class TextSearchQuery_Parser
         
         while ( ($expr = $this->get_expr())
                 || ($expr = $this->get_word($accept_as_words)) ) {
-            
             $list[] = $expr;
         }
 
@@ -508,7 +664,7 @@ class TextSearchQuery_Parser
     
 
     function get_atom() {
-        if ($word = $this->get_word())
+        if ($word = $this->get_word(TSQ_ALLWORDS))
             return $word;
 
         $savedpos = $this->lexer->tell();
@@ -524,16 +680,22 @@ class TextSearchQuery_Parser
         return false;
     }
 
-    function get_word($accept = TSQ_TOK_WORD) {
-        if ( ($word = $this->lexer->get($accept)) )
-            return new TextSearchQuery_node_word($word);
+    function get_word($accept = TSQ_ALLWORDS) {
+        foreach (array("WORD","STARTS_WITH","ENDS_WITH","EXACT",
+                       "REGEX","REGEX_GLOB","REGEX_PCRE","ALL") as $tok) {
+            $const = constant("TSQ_TOK_".$tok);
+            if ( $accept & $const and ($word = $this->lexer->get($const)) ) {
+                $classname = "TextSearchQuery_node_".strtolower($tok);
+                return new $classname($word);
+            }
+        }
         return false;
     }
 }
 
 class TextSearchQuery_Lexer {
-    function TextSearchQuery_Lexer ($query_str) {
-        $this->tokens = $this->tokenize($query_str);
+    function TextSearchQuery_Lexer ($query_str, $case_exact=false, $regex=TSQ_REGEX_AUTO) {
+        $this->tokens = $this->tokenize($query_str, $case_exact, $regex);
         $this->pos = 0;
     }
 
@@ -548,31 +710,89 @@ class TextSearchQuery_Lexer {
     function eof() {
         return $this->pos == count($this->tokens);
     }
-
-    function tokenize($string) {
+    
+    /**
+     * TODO: support more regex styles, esp. prefer the forced ones over auto
+     * re: and // stuff
+     */
+    function tokenize($string, $case_exact=false, $regex=TSQ_REGEX_AUTO) {
         $tokens = array();
-        $buf = strtolower(ltrim($string));
+        $buf = $case_exact ? ltrim($string) : strtolower(ltrim($string));
         while (!empty($buf)) {
-            if (preg_match('/^(and|or)\b\s*/', $buf, $m)) {
-                $val = $m[1];
+            if (preg_match('/^(and|or)\b\s*/i', $buf, $m)) {
+                $val = strtolower($m[1]);
                 $type = TSQ_TOK_BINOP;
             }
-            elseif (preg_match('/^(-|not\b)\s*/', $buf, $m)) {
-                $val = $m[1];
+            elseif (preg_match('/^(-|not\b)\s*/i', $buf, $m)) {
+                $val = strtolower($m[1]);
                 $type = TSQ_TOK_NOT;
             }
             elseif (preg_match('/^([()])\s*/', $buf, $m)) {
                 $val = $m[1];
                 $type = $m[1] == '(' ? TSQ_TOK_LPAREN : TSQ_TOK_RPAREN;
             }
+            
+            // * => ALL
+            elseif ($regex & (TSQ_REGEX_AUTO|TSQ_REGEX_POSIX|TSQ_REGEX_GLOB)
+                    and preg_match('/^\*\s*/', $buf, $m)) {
+                $val = "*";
+                $type = TSQ_TOK_ALL;
+            }
+            // .* => ALL
+            elseif ($regex & (TSQ_REGEX_PCRE)
+                    and preg_match('/^\.\*\s*/', $buf, $m)) {
+                $val = ".*";
+                $type = TSQ_TOK_ALL;
+            }
+            // % => ALL
+            elseif ($regex & (TSQ_REGEX_SQL)
+                    and preg_match('/^%\s*/', $buf, $m)) {
+                $val = "%";
+                $type = TSQ_TOK_ALL;
+            }
+            
+            // ^word
+            elseif ($regex & (TSQ_REGEX_AUTO|TSQ_REGEX_POSIX|TSQ_REGEX_PCRE)
+                    and preg_match('/^\^([^-()][^()\s]*)\s*/', $buf, $m)) {
+                $val = $m[1];
+                $type = TSQ_TOK_STARTS_WITH;
+            }
+            // word*
+            elseif ($regex & (TSQ_REGEX_AUTO|TSQ_REGEX_POSIX|TSQ_REGEX_GLOB)
+                    and preg_match('/^([^-()][^()\s]*)\*\s*/', $buf, $m)) {
+                $val = $m[1];
+                $type = TSQ_TOK_STARTS_WITH;
+            }
+            // *word
+            elseif ($regex & (TSQ_REGEX_AUTO|TSQ_REGEX_POSIX|TSQ_REGEX_GLOB)
+                    and preg_match('/^\*([^-()][^()\s]*)\s*/', $buf, $m)) {
+                $val = $m[1];
+                $type = TSQ_TOK_ENDS_WITH;
+            }
+            // word$
+            elseif ($regex & (TSQ_REGEX_AUTO|TSQ_REGEX_POSIX|TSQ_REGEX_PCRE)
+                    and preg_match('/^([^-()][^()\s]*)\$\s*/', $buf, $m)) {
+                $val = $m[1];
+                $type = TSQ_TOK_ENDS_WITH;
+            }
+            // ^word$
+            elseif ($regex & (TSQ_REGEX_AUTO|TSQ_REGEX_POSIX|TSQ_REGEX_PCRE)
+                    and preg_match('/^\^([^-()][^()\s]*)\$\s*/', $buf, $m)) {
+                $val = $m[1];
+                $type = TSQ_TOK_EXACT;
+            }
+            
+            // "words "
             elseif (preg_match('/^ " ( (?: [^"]+ | "" )* ) " \s*/x', $buf, $m)) {
                 $val = str_replace('""', '"', $m[1]);
                 $type = TSQ_TOK_WORD;
             }
+            // 'words '
             elseif (preg_match("/^ ' ( (?:[^']+|'')* ) ' \s*/x", $buf, $m)) {
                 $val = str_replace("''", "'", $m[1]);
                 $type = TSQ_TOK_WORD;
             }
+            // word
             elseif (preg_match('/^([^-()][^()\s]*)\s*/', $buf, $m)) {
                 $val = $m[1];
                 $type = TSQ_TOK_WORD;
@@ -582,6 +802,17 @@ class TextSearchQuery_Lexer {
                 break;
             }
             $buf = substr($buf, strlen($m[0]));
+
+            /* refine the simple parsing from above: bla*bla, bla?bla, ...
+            if ($regex and $type == TSQ_TOK_WORD) {
+            	if (substr($val,0,1) == "^")
+            	    $type = TSQ_TOK_STARTS_WITH;
+            	elseif (substr($val,0,1) == "*")
+            	    $type = TSQ_TOK_ENDS_WITH;
+            	elseif (substr($val,-1,1) == "*")
+            	    $type = TSQ_TOK_STARTS_WITH;
+            }
+            */
             $tokens[] = array($type, $val);
         }
         return $tokens;
@@ -607,4 +838,4 @@ class TextSearchQuery_Lexer {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
+?>
\ No newline at end of file
