@@ -1,5 +1,5 @@
 <?php
-rcs_id('$Id: cvs.php,v 1.12 2003/12/07 19:27:53 carstenklapp Exp $');
+rcs_id('$Id: cvs.php,v 1.25 2005/09/11 13:23:21 rurban Exp $');
 /**
  * Backend for handling CVS repository. 
  *
@@ -85,10 +85,10 @@ extends WikiDB_backend
         }
         $this->_debug_file = $dbparam{CVS_DEBUG_FILE};
 
-        if ( !($dbparam{CVS_CHECK_FOR_REPOSITORY}
-                && is_dir( $this->_repository )
-                && is_dir( $this->_repository . "/CVSROOT" )
-                && is_dir( $this->_repository . "/" . $this->_module_name ))) {
+        if ( $dbparam{CVS_CHECK_FOR_REPOSITORY}
+             && !( is_dir( $this->_repository )
+                   && is_dir( $this->_repository . "/CVSROOT" )
+                   && is_dir( $this->_repository . "/" . $this->_module_name ))) {
 
             $this->_cvsDebug( sprintf("Creating new repository [%s]", $this->_repository) );
 
@@ -306,11 +306,17 @@ extends WikiDB_backend
         return $rVal;
     }
 
+    /**
+     * See ADODB for a better delete_page(), which can be undone and is seen in RecentChanges.
+     * See backend.php
+     */
+    //function delete_page($pagename) { $this->purge_page($pagename); }
+
     /**
      * This returns false if page was not deleted or could not be deleted
      * else return true.
      */
-    function delete_page($pagename) 
+    function purge_page($pagename) 
     {
         $this->_cvsDebug( "delete_page [$pagename]") ;
         $filename = $this->_docDir . "/" . $pagename;
@@ -328,6 +334,26 @@ extends WikiDB_backend
         return true;
     }
 
+    /**
+     * For now delete and create a new one.
+     *
+     * This returns false if page was not renamed,
+     * else return true.
+     */
+    function rename_page($pagename, $to) 
+    {
+        $this->_cvsDebug( "rename_page [$pagename,$to]") ;
+	$data = get_pagedata($pagename);
+	if (isset($data['pagename']))
+	  $data['pagename'] = $to;
+	//$version = $this->get_latest_version($pagename);
+	//$vdata = get_versiondata($pagename, $version, 1);
+        //$data[CMD_CONTENT] = $vdata[CMD_CONTENT];
+	$this->delete_page($pagename);
+	$this->update_pagedata($to, $data);
+	return true;
+    }
+
     function delete_versiondata($pagename, $version) 
     {
         // TODO: Not Implemented.
@@ -361,7 +387,8 @@ extends WikiDB_backend
         $this->_writeMetaInfo( $pagename, $megaHash );
     }
 
-    function get_links($pagename, $reversed) 
+    function get_links($pagename, $reversed=true, $include_empty=false,
+                       $sortby=false, $limit=false, $exclude=false)
     {
         // TODO: ignores the $reversed argument and returns
         // TODO: the value of _links_ attribute of the meta information
@@ -373,21 +400,20 @@ extends WikiDB_backend
         return $megaHash[CMD_LINK_ATT];
     }
 
-    function get_all_revisions($pagename) 
-    {
+    /* function get_all_revisions($pagename) {
         // TODO: should replace this with something more efficient
         include_once('lib/WikiDB/backend/dumb/AllRevisionsIter.php');
         return new WikiDB_backend_dumb_AllRevisionsIter($this, $pagename);
-    }
+    } */
 
-    function get_all_pages($include_defaulted) 
+    function get_all_pages($include_empty=false, $sortby=false, $limit=false) 
     {
-        // FIXME: this ignores the include_defaulted parameter.
-        return new Cvs_Backend_Array_Iterator( 
+        // FIXME: this ignores the parameters.
+        return new Cvs_Backend_Array_Iterator(
                               $this->_getAllFileNamesInDir( $this->_docDir ));
     }
 
-    function text_search($search = '', $fullsearch = false) 
+    function text_search($search = '', $fullsearch = false, $orderby=false, $limit=false, $exclude=false) 
     {
         if ( $fullsearch ) {
             return new Cvs_Backend_Full_Search_Iterator(
@@ -401,18 +427,17 @@ extends WikiDB_backend
         }
     }
 
-    function most_popular($limit) 
-    {
+    function most_popular($limit, $sortby='') {
         // TODO: needs to be tested ...
         $mp = $this->_getMostPopular();
-        if($limit < 0){
-		asort ($mp, SORT_NUMERIC);
-		$limit = -$limit;
-		} else {
-        arsort( $mp, SORT_NUMERIC );
-		}
+        if ($limit < 0){
+            asort ($mp, SORT_NUMERIC);
+            $limit = -$limit;
+        } else {
+            arsort( $mp, SORT_NUMERIC );
+        }
         $returnVal = array();
-
+        
         while ( (list($key, $val) = each($a)) && $limit > 0 ) {
             $returnVal[] = $key;
             $limit--;
@@ -431,15 +456,15 @@ extends WikiDB_backend
         $mr = $this->_getMostRecent();
         $rev = false;
         $returnVal = array();
-		if ( isset( $params['limit'] ) ) {
+        if ( isset( $params['limit'] ) ) {
             $limit = $params['limit'];
-			$rev = $limit < 0;
+            $rev = $limit < 0;
+        }
+        if ($rev){
+            arsort( $mr, SORT_NUMERIC );
+        } else {
+            asort( $mr, SORT_NUMERIC );
         }
-        if($rev){
-		arsort( $mp, SORT_NUMERIC );
-		} else {
-		asort( $mr, SORT_NUMERIC );
-		}
         if ( isset( $limit ) ) {
             while ( (list($key, $val) = each($a)) && $limit > 0 ) {
                 $returnVal[] = $key;
@@ -448,10 +473,10 @@ extends WikiDB_backend
         } else if ( isset( $params['since'] ) ) {
             while ( (list($key, $val) = each($a)) ) {
                 
-					if ( $val > $params['since'] ) {
-                    	$returnVal[] = $key;
-                    }
-			}
+                if ( $val > $params['since'] ) {
+                    $returnVal[] = $key;
+                }
+            }
         }
 
         return new Cvs_Backend_Array_Iterator( $returnVal );
@@ -908,6 +933,10 @@ extends WikiDB_backend_iterator
         return false;
     }
 
+    function count() {
+    	return count($this->_array);
+    }
+
     function free()
     {
         unset( $this->_array );
@@ -983,6 +1012,12 @@ extends Cvs_Backend_Array_Iterator
     }
 }
 
-
-
+// (c-file-style: "gnu")
+// Local Variables:
+// mode: php
+// tab-width: 8
+// c-basic-offset: 4
+// c-hanging-comment-ender-p: nil
+// indent-tabs-mode: nil
+// End:   
 ?>
