@@ -1,5 +1,5 @@
 <?php //-*-php-*-
-rcs_id('$Id: WikiPlugin.php,v 1.33 2003/09/13 22:20:55 carstenklapp Exp $');
+rcs_id('$Id: WikiPlugin.php,v 1.58 2005/02/12 17:22:17 rurban Exp $');
 
 class WikiPlugin
 {
@@ -87,46 +87,86 @@ class WikiPlugin
     function getVersion() {
         return _("n/a");
         //return preg_replace("/[Revision: $]/", '',
-        //                    "\$Revision: 1.33 $");
+        //                    "\$Revision: 1.58 $");
     }
 
-    function getArgs($argstr, $request=false, $defaults = false) {
-        if ($defaults === false)
+    function getArgs($argstr, $request=false, $defaults=false) {
+        if ($defaults === false) {
             $defaults = $this->getDefaultArguments();
-
+        }
+        //Fixme: on POST argstr is empty
         list ($argstr_args, $argstr_defaults) = $this->parseArgStr($argstr);
         $args = array();
-        foreach ($defaults as $arg => $default_val) {
-            if (isset($argstr_args[$arg]))
+        if (!empty($defaults))
+          foreach ($defaults as $arg => $default_val) {
+            if (isset($argstr_args[$arg])) {
                 $args[$arg] = $argstr_args[$arg];
-            elseif ( $request and ($argval = $request->getArg($arg)) !== false )
+            } elseif ( $request and ($argval = $request->getArg($arg)) !== false ) {
                 $args[$arg] = $argval;
-            elseif (isset($argstr_defaults[$arg]))
+            } elseif (isset($argstr_defaults[$arg])) {
                 $args[$arg] = (string) $argstr_defaults[$arg];
-            else
+            } else {
                 $args[$arg] = $default_val;
-
-            if ($request)
+            }
+            // expand [arg]
+            if ($request and is_string($args[$arg]) and strstr($args[$arg], "[")) {
                 $args[$arg] = $this->expandArg($args[$arg], $request);
+            }
 
             unset($argstr_args[$arg]);
             unset($argstr_defaults[$arg]);
         }
 
         foreach (array_merge($argstr_args, $argstr_defaults) as $arg => $val) {
-            trigger_error(sprintf(_("argument '%s' not declared by plugin"),
-                                  $arg), E_USER_NOTICE);
+            if ($request and $request->getArg('pagename') == _("PhpWikiAdministration") 
+                and $arg == 'overwrite') // silence this warning
+                ;
+            else
+                trigger_error(sprintf(_("argument '%s' not declared by plugin."),
+                                      $arg), E_USER_NOTICE);
         }
 
+        // add special handling of pages and exclude args to accept <! plugin-list !>
+        // and split explodePageList($args['exclude']) => array()
+        // TODO : handle p[] pagehash
+        foreach (array('pages', 'exclude') as $key) {
+            if (!empty($args[$key]) and array_key_exists($key, $defaults))
+                $args[$key] = is_string($args[$key])
+                    ? explodePageList($args[$key])
+                    : $args[$key]; // <! plugin-list !>
+        }
+
+        // always override sortby,limit from the REQUEST. ignore defaults if defined as such.
+        foreach (array('sortby', 'limit') as $key) {
+            if (array_key_exists($key, $defaults)) {
+                if ($val = $request->getArg($key))
+                    $args[$key] = $val;
+                elseif (!empty($args[$key])) {
+                    $GLOBALS['request']->setArg($key, $args[$key]);
+                }
+            }
+        }
         return $args;
     }
 
-    function expandArg($argval, $request) {
-        return preg_replace('/\[(\w[\w\d]*)\]/e', '$request->getArg("$1")',
-                            $argval);
+    // Patch by Dan F:
+    // Expand [arg] to $request->getArg("arg") unless preceded by ~
+    function expandArg($argval, &$request) {
+        // return preg_replace('/\[(\w[\w\d]*)\]/e', '$request->getArg("$1")',
+        // Replace the arg unless it is preceded by a ~
+        $ret = preg_replace('/([^~]|^)\[(\w[\w\d]*)\]/e',
+                            '"$1" . $request->getArg("$2")',
+                           $argval);
+        // Ditch the ~ so later versions can be expanded if desired
+        return preg_replace('/~(\[\w[\w\d]*\])/', '$1', $ret);
     }
 
     function parseArgStr($argstr) {
+        $args = array();
+        $defaults = array();
+	if (empty($argstr))
+            return array($args,$defaults);
+            
         $arg_p = '\w+';
         $op_p = '(?:\|\|)?=';
         $word_p = '\S+';
@@ -137,9 +177,23 @@ class WikiPlugin
         $gt_p = "_\\( $opt_ws $qq_p $opt_ws \\)";
         $argspec_p = "($arg_p) $opt_ws ($op_p) $opt_ws (?: $qq_p|$q_p|$gt_p|($word_p))";
 
-        $args = array();
-        $defaults = array();
-
+        // handle plugin-list arguments seperately
+        $plugin_p = '<!plugin-list\s+\w+.*?!>';
+        while (preg_match("/^($arg_p) $opt_ws ($op_p) $opt_ws ($plugin_p) $opt_ws/x", $argstr, $m)) {
+            @ list(,$arg, $op, $plugin_val) = $m;
+            $argstr = substr($argstr, strlen($m[0]));
+            $loader = new WikiPluginLoader();
+            $markup = null;
+            $basepage = null;
+            $plugin_val = preg_replace(array("/^<!/","/!>$/"),array("<?","?>"),$plugin_val);
+            $val = $loader->expandPI($plugin_val, $GLOBALS['request'], $markup, $basepage);
+            if ($op == '=') {
+                $args[$arg] = $val; // comma delimited pagenames or array()?
+            } else {
+                assert($op == '||=');
+                $defaults[$arg] = $val;
+            }
+        }
         while (preg_match("/^$opt_ws $argspec_p $opt_ws/x", $argstr, $m)) {
             @ list(,$arg,$op,$qq_val,$q_val,$gt_val,$word_val) = $m;
             $argstr = substr($argstr, strlen($m[0]));
@@ -182,11 +236,25 @@ class WikiPlugin
                               $argstr), E_USER_NOTICE);
     }
 
+    /* handle plugin-list argument: use run(). */
+    function makeList($plugin_args, $request, $basepage) {
+        $dbi = $request->getDbh();
+        $pagelist = $this->run($dbi, $plugin_args, $request, $basepage);
+        $list = array();
+        if (is_object($pagelist) and isa($pagelist, 'PageList')) {
+            // table or list?
+            foreach ($pagelist->_pages as $page) {
+            	$list[] = $page->getName();
+            }
+        }
+        return $list;
+    }
+
     function getDefaultLinkArguments() {
-        return array('targetpage' => $this->getName(),
-                     'linktext' => $this->getName(),
+        return array('targetpage'  => $this->getName(),
+                     'linktext'    => $this->getName(),
                      'description' => $this->getDescription(),
-                     'class' => 'wikiaction');
+                     'class'       => 'wikiaction');
     }
 
     function makeLink($argstr, $request) {
@@ -215,17 +283,18 @@ class WikiPlugin
     function getDefaultFormArguments() {
         return array('targetpage' => $this->getName(),
                      'buttontext' => $this->getName(),
-                     'class' => 'wikiaction',
-                     'method' => 'get',
-                     'textinput' => 's',
-                     'description' => $this->getDescription(),
-                     'formsize' => 30);
+                     'class'      => 'wikiaction',
+                     'method'     => 'get',
+                     'textinput'  => 's',
+                     'description'=> $this->getDescription(),
+                     'formsize'   => 30);
     }
     
     function makeForm($argstr, $request) {
         $form_defaults = $this->getDefaultFormArguments();
-        $defaults = array_merge($this->getDefaultArguments(),
-                                $form_defaults);
+        $defaults = array_merge($form_defaults, 
+                                array('start_debug' => $request->getArg('start_debug')),
+        			$this->getDefaultArguments());
     
         $args = $this->getArgs($argstr, $request, $defaults);
         $plugin = $this->getName();
@@ -234,13 +303,19 @@ class WikiPlugin
     
         $form = HTML::form(array('action' => WikiURL($args['targetpage']),
                                  'method' => $args['method'],
-                                 'class' => $args['class'],
-                                 'accept-charset' => CHARSET));
+                                 'class'  => $args['class'],
+                                 'accept-charset' => $GLOBALS['charset']));
         if (! USE_PATH_INFO ) {
             $pagename = $request->get('pagename');
-            $form->pushContent(HTML::input(array('type' => 'hidden', 'name' => 'pagename', 
+            $form->pushContent(HTML::input(array('type' => 'hidden', 
+                                                 'name' => 'pagename', 
                                                  'value' => $args['targetpage'])));
         }
+        if ($args['targetpage'] != $this->getName()) {
+            $form->pushContent(HTML::input(array('type' => 'hidden', 
+                                                 'name' => 'action', 
+                                                 'value' => $this->getName())));
+        }
         $contents = HTML::div();
         $contents->setAttr('class', $args['class']);
     
@@ -280,10 +355,23 @@ class WikiPlugin
             $contents->pushContent(HTML::input(array('type' => 'submit',
                                                      'class' => 'button',
                                                      'value' => $args['buttontext'])));
-    
         $form->pushContent($contents);
         return $form;
     }
+
+    // box is used to display a fixed-width, narrow version with common header
+    function box($args=false, $request=false, $basepage=false) {
+        if (!$request) $request =& $GLOBALS['request'];
+        $dbi = $request->getDbh();
+        return $this->makeBox('', $this->run($dbi, $args, $request, $basepage));
+    }
+
+    function makeBox($title, $body) {
+        if (!$title) $title = $this->getName();
+        return HTML::div(array('class'=>'box'),
+                         HTML::div(array('class'=>'box-title'), $title),
+                         HTML::div(array('class'=>'box-data'), $body));
+    }
     
     function error ($message) {
         return HTML::div(array('class' => 'errors'),
@@ -298,12 +386,36 @@ class WikiPlugin
         $html[] = HTML::pre($this->_pi);
         return HTML::div(array('class' => 'disabled-plugin'), $html);
     }
+
+    // TODO: Not really needed, since our plugins generally initialize their own 
+    // PageList object, which accepts options['types'].
+    // Register custom PageList types for special plugins, like 
+    // 'hi_content' for WikiAdminSearcheplace, 'renamed_pagename' for WikiAdminRename, ...
+    function addPageListColumn ($array) {
+        global $customPageListColumns;
+        if (empty($customPageListColumns)) $customPageListColumns = array();
+        foreach ($array as $column => $obj) {
+            $customPageListColumns[$column] = $obj;
+        }
+    }
+    
+    // provide a sample usage text for automatic edit-toolbar insertion
+    function getUsage() {
+        $args = $this->getDefaultArguments();
+        $string = '<'.'?plugin '.$this->getName().' ';
+        if ($args) {
+            foreach ($args as $key => $value) {
+                $string .= ($key."||=".(string)$value." ");
+            }
+        }
+        return $string . '?'.'>';
+    }
 }
 
 class WikiPluginLoader {
     var $_errors;
 
-    function expandPI($pi, &$request, $basepage=false) {
+    function expandPI($pi, &$request, &$markup, $basepage=false) {
         if (!($ppi = $this->parsePi($pi)))
             return false;
         list($pi_name, $plugin, $plugin_args) = $ppi;
@@ -317,6 +429,9 @@ class WikiPluginLoader {
             case 'plugin':
                 // FIXME: change API for run() (no $dbi needed).
                 $dbi = $request->getDbh();
+                // pass the parsed CachedMarkup context in dbi to the plugin 
+                // to be able to know about itself, or even to change the markup XmlTree (CreateToc)
+                $dbi->_markup = &$markup; 
                 // FIXME: could do better here...
                 if (! $plugin->managesValidators()) {
                     // Output of plugin (potentially) depends on
@@ -337,6 +452,8 @@ class WikiPluginLoader {
                                                      '%weak' => true));
                 }
                 return $plugin->run($dbi, $plugin_args, $request, $basepage);
+            case 'plugin-list':
+                return $plugin->makeList($plugin_args, $request, $basepage);
             case 'plugin-link':
                 return $plugin->makeLink($plugin_args, $request);
             case 'plugin-form':
@@ -356,7 +473,7 @@ class WikiPluginLoader {
     }
     
     function parsePI($pi) {
-        if (!preg_match('/^\s*<\?(plugin(?:-form|-link)?)\s+(\w+)\s*(.*?)\s*\?>\s*$/s', $pi, $m))
+        if (!preg_match('/^\s*<\?(plugin(?:-form|-link|-list)?)\s+(\w+)\s*(.*?)\s*\?>\s*$/s', $pi, $m))
             return $this->_error(sprintf("Bad %s", 'PI'));
 
         list(, $pi_name, $plugin_name, $plugin_args) = $m;
@@ -365,7 +482,7 @@ class WikiPluginLoader {
         return array($pi_name, $plugin, $plugin_args);
     }
     
-    function getPlugin($plugin_name, $pi) {
+    function getPlugin($plugin_name, $pi=false) {
         global $ErrorManager;
 
         // Note that there seems to be no way to trap parse errors
@@ -373,21 +490,23 @@ class WikiPluginLoader {
         $plugin_source = "lib/plugin/$plugin_name.php";
 
         $ErrorManager->pushErrorHandler(new WikiMethodCb($this, '_plugin_error_filter'));
-        // $include_failed = !@include_once("lib/plugin/$plugin_name.php");
-        $include_failed = !include_once("lib/plugin/$plugin_name.php");
-        $ErrorManager->popErrorHandler();
-
         $plugin_class = "WikiPlugin_$plugin_name";
         if (!class_exists($plugin_class)) {
-            if ($include_failed)
-                return $this->_error(sprintf(_("Include of '%s' failed"),
-                                             $plugin_source));
-            return $this->_error(sprintf("%s: no such class", $plugin_class));
+            // $include_failed = !@include_once("lib/plugin/$plugin_name.php");
+            $include_failed = !include_once("lib/plugin/$plugin_name.php");
+            $ErrorManager->popErrorHandler();
+            
+            if (!class_exists($plugin_class)) {
+                if ($include_failed)
+                    return $this->_error(sprintf(_("Include of '%s' failed."),
+                                                 $plugin_source));
+                return $this->_error(sprintf(_("%s: no such class"), $plugin_class));
+            }
         }
-
+        $ErrorManager->popErrorHandler();
         $plugin = new $plugin_class;
         if (!is_subclass_of($plugin, "WikiPlugin"))
-            return $this->_error(sprintf("%s: not a subclass of WikiPlugin",
+            return $this->_error(sprintf(_("%s: not a subclass of WikiPlugin."),
                                          $plugin_class));
 
         $plugin->_pi = $pi;
