@@ -1,5 +1,5 @@
 <?php //-*-php-*-
-rcs_id('$Id: WikiUser.php,v 1.45 2003/12/09 20:00:43 carstenklapp Exp $');
+rcs_id('$Id: WikiUser.php,v 1.65 2005/06/05 05:38:02 rurban Exp $');
 
 // It is anticipated that when userid support is added to phpwiki,
 // this object will hold much more information (e-mail,
@@ -15,12 +15,16 @@ rcs_id('$Id: WikiUser.php,v 1.45 2003/12/09 20:00:43 carstenklapp Exp $');
 //       serialized string.
 //       If no homepage, fallback to prefs in cookie as in 1.3.3.
 
+define('WIKIAUTH_FORBIDDEN', -1); // Completely not allowed.
 define('WIKIAUTH_ANON', 0);
 define('WIKIAUTH_BOGO', 1);     // any valid WikiWord is enough
 define('WIKIAUTH_USER', 2);     // real auth from a database/file/server.
 
-define('WIKIAUTH_ADMIN', 10);
-define('WIKIAUTH_FORBIDDEN', 11); // Completely not allowed.
+define('WIKIAUTH_ADMIN', 10);  // Wiki Admin
+define('WIKIAUTH_UNOBTAINABLE', 100);  // Permissions that no user can achieve
+
+if (!defined('COOKIE_EXPIRATION_DAYS')) define('COOKIE_EXPIRATION_DAYS', 365);
+if (!defined('COOKIE_DOMAIN'))          define('COOKIE_DOMAIN', '/');
 
 $UserPreferences = array(
                          'userid'        => new _UserPreference(''), // really store this also?
@@ -34,9 +38,28 @@ $UserPreferences = array(
                          'noLinkIcons'   => new _UserPreference_bool(),
                          'editHeight'    => new _UserPreference_int(22, 5, 80),
                          'timeOffset'    => new _UserPreference_numeric(0, -26, 26),
-                         'relativeDates' => new _UserPreference_bool()
+                         'relativeDates' => new _UserPreference_bool(),
+                         'googleLink'    => new _UserPreference_bool(), // 1.3.10
+                         'doubleClickEdit' => new _UserPreference_bool(), // 1.3.11
                          );
 
+function WikiUserClassname() {
+    return 'WikiUser';
+}
+
+function UpgradeUser ($olduser, $user) {
+    if (isa($user,'WikiUser') and isa($olduser,'WikiUser')) {
+        // populate the upgraded class with the values from the old object
+        foreach (get_object_vars($olduser) as $k => $v) {
+            $user->$k = $v;	
+        }
+        $GLOBALS['request']->_user = $user;
+        return $user;
+    } else {
+        return false;
+    }
+}
+
 /**
 * 
 */
@@ -107,6 +130,10 @@ class WikiUser {
         return false;
     }
 
+    function UserName() {
+        return $this->_userid;
+    }
+
     function getId () {
         return ( $this->isSignedIn()
                  ? $this->_userid
@@ -124,7 +151,7 @@ class WikiUser {
     }
 
     function isAuthenticated () {
-        return $this->_level >= WIKIAUTH_USER;
+        return $this->_level >= WIKIAUTH_BOGO;
     }
 
     function isAdmin () {
@@ -135,6 +162,12 @@ class WikiUser {
         return $this->_level >= $require_level;
     }
 
+    function isValidName ($userid = false) {
+        if (!$userid)
+            $userid = $this->_userid;
+        return preg_match("/^[\w\.@\-]+$/",$userid) and strlen($userid) < 32;
+    }
+
     function AuthCheck ($postargs) {
         // Normalize args, and extract.
         $keys = array('userid', 'passwd', 'require_level', 'login', 'logout',
@@ -151,6 +184,9 @@ class WikiUser {
         elseif (!$login && !$userid)
             return false;       // Nothing to do?
 
+        if (!$this->isValidName($userid))
+            return _("Invalid username.");
+
         $authlevel = $this->_pwcheck($userid, $passwd);
         if (!$authlevel)
             return _("Invalid password or userid.");
@@ -158,9 +194,7 @@ class WikiUser {
             return _("Insufficient permissions.");
 
         // Successful login.
-        $user = new WikiUser($this->_request);
-        $user->_userid = $userid;
-        $user->_level = $authlevel;
+        $user = new WikiUser($this->_request, $userid, $authlevel);
         return $user;
     }
 
@@ -181,9 +215,10 @@ class WikiUser {
                               compact('pagename', 'userid', 'require_level',
                                       'fail_message', 'pass_required'));
         if ($seperate_page) {
-            $top = new Template('html', $request,
-                                array('TITLE' => _("Sign In")));
-            return $top->printExpansion($login);
+            $request->discardOutput();
+            $page = $request->getPage($pagename);
+            $revision = $page->getCurrentRevision();
+            return GeneratePage($login,_("Sign In"),$revision);
         } else {
             return $login;
         }
@@ -210,7 +245,7 @@ class WikiUser {
                     // maybe we forgot to enable ENCRYPTED_PASSWD?
                     if ( function_exists('crypt')
                          && crypt($passwd, ADMIN_PASSWD) == ADMIN_PASSWD ) {
-                        trigger_error(_("You forgot to set ENCRYPTED_PASSWD to true. Please update your /index.php"),
+                        trigger_error(_("You forgot to set ENCRYPTED_PASSWD to true. Please update your config/config.ini"),
                                       E_USER_WARNING);
                         return WIKIAUTH_ADMIN;
                     }
@@ -229,6 +264,10 @@ class WikiUser {
         // WikiDB_User DB/File Authentication from $DBAuthParams
         // Check if we have the user. If not try other methods.
         if (ALLOW_USER_LOGIN) { // && !empty($passwd)) {
+            if (!$this->isValidName($userid)) {
+                trigger_error(_("Invalid username."), E_USER_WARNING);
+                return false;
+            }
             $request = $this->_request;
             // first check if the user is known
             if ($this->exists($userid)) {
@@ -236,12 +275,12 @@ class WikiUser {
                 return ($this->checkPassword($passwd)) ? WIKIAUTH_USER : false;
             } else {
                 // else try others such as LDAP authentication:
-                if (ALLOW_LDAP_LOGIN && !empty($passwd)) {
+                if (ALLOW_LDAP_LOGIN && defined(LDAP_AUTH_HOST) && !empty($passwd) && !strstr($userid,'*')) {
                     if ($ldap = ldap_connect(LDAP_AUTH_HOST)) { // must be a valid LDAP server!
                         $r = @ldap_bind($ldap); // this is an anonymous bind
                         $st_search = "uid=$userid";
                         // Need to set the right root search information. see ../index.php
-                        $sr = ldap_search($ldap, LDAP_AUTH_SEARCH,
+                        $sr = ldap_search($ldap, LDAP_BASE_DN,
                                           "$st_search");
                         $info = ldap_get_entries($ldap, $sr); // there may be more hits with this userid. try every
                         for ($i = 0; $i < $info["count"]; $i++) {
@@ -261,7 +300,7 @@ class WikiUser {
                 }
                 // imap authentication. added by limako
                 if (ALLOW_IMAP_LOGIN && !empty($passwd)) {
-                    $mbox = @imap_open( "{" . IMAP_AUTH_HOST . ":143}",
+                    $mbox = @imap_open( "{" . IMAP_AUTH_HOST . "}INBOX",
                                         $userid, $passwd, OP_HALFOPEN );
                     if($mbox) {
                         imap_close($mbox);
@@ -294,6 +333,7 @@ class WikiUser {
 
         // before we get his prefs we should check if he is signed in
         if (USE_PREFS_IN_PAGE && $this->homePage()) { // in page metadata
+            // old array
             if ($pref = $this->_homepage->get('pref')) {
                 //trigger_error("pref=".$pref);//debugging
                 $prefs = unserialize($pref);
@@ -303,11 +343,15 @@ class WikiUser {
     }
 
     // No cookies anymore for all prefs, only the userid. PHP creates
-    // a session cookie in memory, which is much more efficient.
+    // a session cookie in memory, which is much more efficient, 
+    // but not persistent. Get persistency with a homepage or DB Prefs
     //
-    // Return the number of changed entries?
+    // Return the number of changed entries
     function setPreferences($prefs, $id_only = false) {
-        // update the id
+        if (!is_object($prefs)) {
+            $prefs = new UserPreferences($prefs);
+        }
+        // update the session and id
         $this->_request->setSessionVar('wiki_prefs', $prefs);
         // $this->_request->setCookieVar('WIKI_PREFS2', $this->_prefs, 365);
         // simple unpacked cookie
@@ -319,7 +363,7 @@ class WikiUser {
             if ($this->isSignedIn()) {
                 if ($this->isAdmin())
                     $prefs->set('passwd', '');
-                // already stored in index.php, and it might be
+                // already stored in config/config.ini, and it might be
                 // plaintext! well oh well
                 if ($homepage = $this->homePage()) {
                     // check for page revision 0
@@ -380,6 +424,10 @@ class WikiUser {
         }
     }
 
+    function hasHomePage() {
+        return !$this->homePage();
+    }
+
     // create user by checking his homepage
     function createUser ($pref, $createDefaultHomepage = true) {
         if ($this->exists())
@@ -463,23 +511,28 @@ class WikiUser {
     }
 
     function changePassword($newpasswd, $passwd2 = false) {
-        if (! $this->mayChangePassword() ) {
+        if (! $this->mayChangePass() ) {
             trigger_error(sprintf("Attempt to change an external password for '%s'. Not allowed!",
                                   $this->_userid), E_USER_ERROR);
-            return;
+            return false;
         }
         if ($passwd2 && $passwd2 != $newpasswd) {
             trigger_error("The second password must be the same as the first to change it",
                           E_USER_ERROR);
-            return;
+            return false;
         }
+        if (!$this->isAuthenticated()) return false;
+
         $prefs = $this->getPreferences();
-        //$oldpasswd = $prefs->get('passwd');
-        $prefs->set('passwd', crypt($newpasswd));
+        if (ENCRYPTED_PASSWD)
+            $prefs->set('passwd', crypt($newpasswd));
+        else 
+            $prefs->set('passwd', $newpasswd);
         $this->setPreferences($prefs);
+        return true;
     }
 
-    function mayChangePassword() {
+    function mayChangePass() {
         // on external DBAuth maybe. on IMAP or LDAP not
         // on internal DBAuth yes
         if (in_array($this->_authmethod, array('IMAP', 'LDAP')))
@@ -610,9 +663,9 @@ extends _UserPreference
     }
 
     function update ($newvalue) {
-        global $Theme;
+        global $WikiTheme;
         include_once($this->_themefile($newvalue));
-        if (empty($Theme))
+        if (empty($WikiTheme))
             include_once($this->_themefile(THEME));
     }
 
@@ -672,12 +725,148 @@ class UserPreferences {
             $this->_prefs[$name] = $newvalue;
     }
 
+    function pack ($nonpacked) {
+        return serialize($nonpacked);
+    }
+    function unpack ($packed) {
+        if (!$packed)
+            return false;
+        if (substr($packed,0,2) == "O:") {
+            // Looks like a serialized object
+            return unserialize($packed);
+        }
+        //trigger_error("DEBUG: Can't unpack bad UserPreferences",
+        //E_USER_WARNING);
+        return false;
+    }
+
     function hash () {
         return hash($this->_prefs);
     }
 }
 
 // $Log: WikiUser.php,v $
+// Revision 1.65  2005/06/05 05:38:02  rurban
+// Default ENABLE_DOUBLECLICKEDIT = false. Moved to UserPreferences
+//
+// Revision 1.64  2005/02/08 13:25:50  rurban
+// encrypt password. fix strict logic.
+// both bugs reported by Mikhail Vladimirov
+//
+// Revision 1.63  2005/01/21 14:07:50  rurban
+// reformatting
+//
+// Revision 1.62  2004/11/21 11:59:16  rurban
+// remove final \n to be ob_cache independent
+//
+// Revision 1.61  2004/10/21 21:02:04  rurban
+// fix seperate page login
+//
+// Revision 1.60  2004/06/15 09:15:52  rurban
+// IMPORTANT: fixed passwd handling for passwords stored in prefs:
+//   fix encrypted usage, actually store and retrieve them from db
+//   fix bogologin with passwd set.
+// fix php crashes with call-time pass-by-reference (references wrongly used
+//   in declaration AND call). This affected mainly Apache2 and IIS.
+//   (Thanks to John Cole to detect this!)
+//
+// Revision 1.59  2004/06/14 11:31:36  rurban
+// renamed global $Theme to $WikiTheme (gforge nameclash)
+// inherit PageList default options from PageList
+//   default sortby=pagename
+// use options in PageList_Selectable (limit, sortby, ...)
+// added action revert, with button at action=diff
+// added option regex to WikiAdminSearchReplace
+//
+// Revision 1.58  2004/06/04 20:32:53  rurban
+// Several locale related improvements suggested by Pierrick Meignen
+// LDAP fix by John Cole
+// reanable admin check without ENABLE_PAGEPERM in the admin plugins
+//
+// Revision 1.57  2004/06/04 12:40:21  rurban
+// Restrict valid usernames to prevent from attacks against external auth or compromise
+// possible holes.
+// Fix various WikiUser old issues with default IMAP,LDAP,POP3 configs. Removed these.
+// Fxied more warnings
+//
+// Revision 1.56  2004/06/03 12:36:03  rurban
+// fix eval warning on signin
+//
+// Revision 1.55  2004/06/03 09:39:51  rurban
+// fix LDAP injection (wildcard in username) detected by Steve Christey, MITRE
+//
+// Revision 1.54  2004/04/29 17:18:19  zorloc
+// Fixes permission failure issues.  With PagePermissions and Disabled Actions when user did not have permission WIKIAUTH_FORBIDDEN was returned.  In WikiUser this was ok because WIKIAUTH_FORBIDDEN had a value of 11 -- thus no user could perform that action.  But WikiUserNew has a WIKIAUTH_FORBIDDEN value of -1 -- thus a user without sufficent permission to do anything.  The solution is a new high value permission level (WIKIAUTH_UNOBTAINABLE) to be the default level for access failure.
+//
+// Revision 1.53  2004/04/10 05:34:35  rurban
+// sf bug#830912
+//
+// Revision 1.52  2004/04/10 02:55:48  rurban
+// fixed old WikiUser
+//
+// Revision 1.51  2004/04/06 20:00:10  rurban
+// Cleanup of special PageList column types
+// Added support of plugin and theme specific Pagelist Types
+// Added support for theme specific UserPreferences
+// Added session support for ip-based throttling
+//   sql table schema change: ALTER TABLE session ADD sess_ip CHAR(15);
+// Enhanced postgres schema
+// Added DB_Session_dba support
+//
+// Revision 1.50  2004/02/26 01:32:03  rurban
+// fixed session login with old WikiUser object. strangely, the errormask gets corruoted to 1, Pear???
+//
+// Revision 1.49  2004/02/15 21:34:37  rurban
+// PageList enhanced and improved.
+// fixed new WikiAdmin... plugins
+// editpage, Theme with exp. htmlarea framework
+//   (htmlarea yet committed, this is really questionable)
+// WikiUser... code with better session handling for prefs
+// enhanced UserPreferences (again)
+// RecentChanges for show_deleted: how should pages be deleted then?
+//
+// Revision 1.48  2004/02/01 09:14:11  rurban
+// Started with Group_Ldap (not yet ready)
+// added new _AuthInfo plugin to help in auth problems (warning: may display passwords)
+// fixed some configurator vars
+// renamed LDAP_AUTH_SEARCH to LDAP_BASE_DN
+// changed PHPWIKI_VERSION from 1.3.8a to 1.3.8pre
+// USE_DB_SESSION defaults to true on SQL
+// changed GROUP_METHOD definition to string, not constants
+// changed sample user DBAuthParams from UPDATE to REPLACE to be able to
+//   create users. (Not to be used with external databases generally, but
+//   with the default internal user table)
+//
+// fixed the IndexAsConfigProblem logic. this was flawed:
+//   scripts which are the same virtual path defined their own lib/main call
+//   (hmm, have to test this better, phpwiki.sf.net/demo works again)
+//
+// Revision 1.47  2004/01/27 23:23:39  rurban
+// renamed ->Username => _userid for consistency
+// renamed mayCheckPassword => mayCheckPass
+// fixed recursion problem in WikiUserNew
+// fixed bogo login (but not quite 100% ready yet, password storage)
+//
+// Revision 1.46  2004/01/26 09:17:48  rurban
+// * changed stored pref representation as before.
+//   the array of objects is 1) bigger and 2)
+//   less portable. If we would import packed pref
+//   objects and the object definition was changed, PHP would fail.
+//   This doesn't happen with an simple array of non-default values.
+// * use $prefs->retrieve and $prefs->store methods, where retrieve
+//   understands the interim format of array of objects also.
+// * simplified $prefs->get() and fixed $prefs->set()
+// * added $user->_userid and class '_WikiUser' portability functions
+// * fixed $user object ->_level upgrading, mostly using sessions.
+//   this fixes yesterdays problems with loosing authorization level.
+// * fixed WikiUserNew::checkPass to return the _level
+// * fixed WikiUserNew::isSignedIn
+// * added explodePageList to class PageList, support sortby arg
+// * fixed UserPreferences for WikiUserNew
+// * fixed WikiPlugin for empty defaults array
+// * UnfoldSubpages: added pagename arg, renamed pages arg,
+//   removed sort arg, support sortby arg
+//
 // Revision 1.45  2003/12/09 20:00:43  carstenklapp
 // Bugfix: The last BogoUserPrefs-bugfix prevented the admin from saving
 // prefs into his own homepage, fixed broken logic. Tightened up BogoUser
@@ -790,4 +979,4 @@ class UserPreferences {
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:
-?>
+?>
\ No newline at end of file
