@@ -1,6 +1,7 @@
-<?php 
-// $Id: XmlRpcServer.php,v 1.5 2003/02/21 04:12:05 dairiki Exp $
+<?php
+// $Id: XmlRpcServer.php,v 1.14 2005/07/21 19:27:04 rurban Exp $
 /* Copyright (C) 2002, Lawrence Akka <lakka@users.sourceforge.net>
+ * Copyright (C) 2004, 2005 $ThePhpWikiProgrammingTeam
  *
  * LICENCE
  * =======
@@ -23,15 +24,11 @@
  * LIBRARY USED - POSSIBLE PROBLEMS
  * ================================
  * 
- * This file provides an XML-RPC interface for PhpWiki.  It uses the XML-RPC 
- * library for PHP by Edd Dumbill - see http://xmlrpc.usefulinc.com/php.html 
- * for details.
- *
- * PHP >= 4.1.0 includes experimental support for the xmlrpc-epi c library 
- * written by Dan Libby (see http://uk2.php.net/manual/en/ref.xmlrpc.php).  This
- * is not compiled into PHP by default.  If it *is* compiled into your installation
- * (ie you have --with-xmlrpc) there may well be namespace conflicts with the xml-rpc
- * library used by this code, and you will get errors.
+ * This file provides an XML-RPC interface for PhpWiki. 
+ * It checks for the existence of the xmlrpc-epi c library by Dan Libby 
+ * (see http://uk2.php.net/manual/en/ref.xmlrpc.php), and falls back to 
+ * the slower PHP counterpart XML-RPC library by Edd Dumbill. 
+ * See http://xmlrpc.usefulinc.com/php.html for details.
  * 
  * INTERFACE SPECIFICTION
  * ======================
@@ -40,28 +37,32 @@
  * http://www.ecyrd.com/JSPWiki/Wiki.jsp?page=WikiRPCInterface
  * 
  * See also http://www.usemod.com/cgi-bin/mb.pl?XmlRpc
+ * or http://www.devshed.com/c/a/PHP/Using-XMLRPC-with-PHP/
  * 
- * NB:  All XMLRPC methods should be prefixed with "wiki."
- * eg  wiki.getAllPages
- * 
+ * Note: All XMLRPC methods are automatically prefixed with "wiki."
+ *       eg. "wiki.getAllPages"
 */
 
-// ToDo:  
-//        Remove all warnings from xmlrpc.inc 
-//        Return list of external links in listLinks
- 
+/*
+ToDo:
+        Remove all warnings from xmlrpc.inc 
+        Return list of external links in listLinks
+        Support RSS2 cloud subscription
+Done:
+        Test hwiki.jar xmlrpc interface (java visualization plugin)
+     	Make use of the xmlrpc extension if found. http://xmlrpc-epi.sourceforge.net/
+	Resolved namespace conflicts
+*/
 
 // Intercept GET requests from confused users.  Only POST is allowed here!
 // There is some indication that $HTTP_SERVER_VARS is deprecated in php > 4.1.0
 // in favour of $_Server, but as far as I know, it still works.
-if ($GLOBALS['HTTP_SERVER_VARS']['REQUEST_METHOD'] != "POST")  
+if ($GLOBALS['HTTP_SERVER_VARS']['REQUEST_METHOD'] != "POST")
 {
     die('This is the address of the XML-RPC interface.' .
         '  You must use XML-RPC calls to access information here');
 }
 
-// Include the php XML-RPC library
-
 // All these global declarations make it so that this file
 // (XmlRpcServer.php) can be included within a function body
 // (not in global scope), and things will still work....
@@ -76,14 +77,30 @@ global $xmlrpcName, $xmlrpcVersion;
 global $xmlrpcerruser, $xmlrpcerrxml;
 global $xmlrpc_backslash;
 global $_xh;
-include_once("lib/XMLRPC/xmlrpc.inc");
 
-global $_xmlrpcs_dmap;
-global $_xmlrpcs_debug;
-include_once("lib/XMLRPC/xmlrpcs.inc");
+if (loadPhpExtension('xmlrpc')) { // fast c lib
+    define('XMLRPC_EXT_LOADED', true);
+
+    global $xmlrpc_util_path;
+    $xmlrpc_util_path = dirname(__FILE__)."/XMLRPC/";
+    include_once("lib/XMLRPC/xmlrpc_emu.inc"); 
+    global $_xmlrpcs_debug;
+    include_once("lib/XMLRPC/xmlrpcs_emu.inc");
+
+ } else { // slow php lib
+    define('XMLRPC_EXT_LOADED', true);
+
+    // Include the php XML-RPC library
+    include_once("lib/XMLRPC/xmlrpc.inc");
+
+    global $_xmlrpcs_dmap;
+    global $_xmlrpcs_debug;
+    include_once("lib/XMLRPC/xmlrpcs.inc");
+}
+ 
 
 //  API version
-define ("WIKI_XMLRPC_VERSION", 1);
+define ("WIKI_XMLRPC_VERSION", 2);
 
 /**
  * Helper function:  Looks up a page revision (most recent by default) in the wiki database
@@ -160,10 +177,10 @@ function short_string_decode ($str) {
 /**
  * Get an xmlrpc "No such page" error message
  */
-function NoSuchPage () 
+function NoSuchPage ($pagename='') 
 {
     global $xmlrpcerruser;
-    return new xmlrpcresp(0, $xmlrpcerruser + 1, "No such page");
+    return new xmlrpcresp(0, $xmlrpcerruser + 1, "No such page ".$pagename);
 }
 
 
@@ -184,7 +201,7 @@ $wiki_dmap['getRPCVersionSupported']
 // method.
 function getRPCVersionSupported($params)
 {
-    return new xmlrpcresp(new xmlrpcval(WIKI_XMLRPC_VERSION, "int"));
+    return new xmlrpcresp(new xmlrpcval((integer)WIKI_XMLRPC_VERSION, "int"));
 }
 
 /**
@@ -206,7 +223,7 @@ $wiki_dmap['getRecentChanges']
 function getRecentChanges($params)
 {
     global $request;
-    // Get the first parameter as an ISO 8601 date.  Assume UTC
+    // Get the first parameter as an ISO 8601 date. Assume UTC
     $encoded_date = $params->getParam(0);
     $datetime = iso8601_decode($encoded_date->scalarval(), 1);
     $dbh = $request->getDbh();
@@ -221,10 +238,10 @@ function getRecentChanges($params)
         $version = new xmlrpcval($page->getVersion(), 'int');
 
         // Build an array of xmlrpc structs
-        $pages[] = new xmlrpcval(array('name'=>$name, 
-                                       'lastModified'=>$lastmodified,
-                                       'author'=>$author,
-                                       'version'=>$version),
+        $pages[] = new xmlrpcval(array('name' => $name, 
+                                       'lastModified' => $lastmodified,
+                                       'author' => $author,
+                                       'version' => $version),
                                  'struct');
     } 
     return new xmlrpcresp(new xmlrpcval($pages, "array"));
@@ -245,8 +262,11 @@ function getPage($params)
 {
     $revision = _getPageRevision($params);
 
-    if (! $revision)
-        return NoSuchPage();
+    if (! $revision ) {
+        $ParamPageName = $params->getParam(0);
+        $pagename = short_string_decode($ParamPageName->scalarval());
+        return NoSuchPage($pagename);
+    }
 
     return new xmlrpcresp(long_string($revision->getPackedContent()));
 }
@@ -393,65 +413,295 @@ function listLinks($params)
     $pagename = short_string_decode($ParamPageName->scalarval());
     $dbh = $request->getDbh();
     if (! $dbh->isWikiPage($pagename))
-        return NoSuchPage();
+        return NoSuchPage($pagename);
 
     $page = $dbh->getPage($pagename);
-    /*
-    $linkiterator = $page->getLinks(false);
+    
+    // The fast WikiDB method. below is the slow method which goes through the formatter
+    // NB no clean way to extract a list of external links yet, so
+    // only internal links returned.  i.e. all type 'local'.
+    $linkiterator = $page->getPageLinks();
     $linkstruct = array();
     while ($currentpage = $linkiterator->next()) {
         $currentname = $currentpage->getName();
-        $name = short_string($currentname);
-        // NB no clean way to extract a list of external links yet, so
-        // only internal links returned.  ie all type 'local'.
-        $type = new xmlrpcval('local');
-
         // Compute URL to page
         $args = array();
-        $currentrev = $currentpage->getCurrentRevision();
-        if ($currentrev->hasDefaultContents())
-            $args['action'] = 'edit';
+        // How to check external links?
+        if (!$currentpage->exists()) $args['action'] = 'edit';
 
         // FIXME: Autodetected value of VIRTUAL_PATH wrong,
-        // this make absolute URLs contstructed by WikiURL wrong.
+        // this make absolute URLs constructed by WikiURL wrong.
         // Also, if USE_PATH_INFO is false, WikiURL is wrong
         // due to its use of SCRIPT_NAME.
-        $use_abspath = USE_PATH_INFO && ! preg_match('/RPC2.php$/', VIRTUAL_PATH);
-        $href = new xmlrpcval(WikiURL($currentname, $args, $use_abspath));
-            
-        $linkstruct[] = new xmlrpcval(array('name'=> $name,
-                                            'type'=> $type,
-                                            'href' => $href),
+        //$use_abspath = USE_PATH_INFO && ! preg_match('/RPC2.php$/', VIRTUAL_PATH);
+        
+        // USE_PATH_INFO must be defined in index.php or config.ini but not before, 
+        // otherwise it is ignored and xmlrpc urls are wrong.
+        // SCRIPT_NAME here is always .../RPC2.php
+        if (USE_PATH_INFO and !$args) {
+            $url = preg_replace('/%2f/i', '/', rawurlencode($currentname));
+        } elseif (!USE_PATH_INFO) {
+            $url = str_replace("/RPC2.php","/index.php", WikiURL($currentname, $args, true));
+        } else {
+            $url = WikiURL($currentname, $args);
+        }
+        $linkstruct[] = new xmlrpcval(array('page'=> short_string($currentname),
+                                            'type'=> new xmlrpcval('local', 'string'),
+                                            'href' => short_string($url)),
                                       "struct");
     }
-    */
-    
+   
+    /*
     $current = $page->getCurrentRevision();
     $content = $current->getTransformedContent();
     $links = $content->getLinkInfo();
-
     foreach ($links as $link) {
         // We used to give an href for unknown pages that
         // included action=edit.  I think that's probably the
         // wrong thing to do.
-        $linkstruct[] = new xmlrpcval(array('name'=> short_string($link->page),
-                                            'type'=> new xmlrpcval($link->type),
-                                            'href' => short_string($link->href)),
+        $linkstruct[] = new xmlrpcval(array('page'=> short_string($link->page),
+                                            'type'=> new xmlrpcval($link->type, 'string'),
+                                            'href' => short_string($link->href),
+                                            //'pageref' => short_string($link->pageref),
+                                            ),
                                       "struct");
     }
-        
+    */
     return new xmlrpcresp(new xmlrpcval ($linkstruct, "array"));
 } 
+
+/** 
+ * struct putPage(String pagename, String content, [String author[, String password]})
+ * returns a struct with elements: 
+ *   code (int): 200 on success, 400 or 401 on failure
+ *   message (string): success or failure message
+ *   version (int): version of new page
+ *
+ * @author: Arnaud Fontaine, Reini Urban
+ */
+$wiki_dmap['putPage']
+= array('signature'     => array(array($xmlrpcStruct, $xmlrpcString, $xmlrpcString, $xmlrpcString, $xmlrpcString)),
+        'documentation' => 'put the raw Wiki text into a page as new version',
+        'function'      => 'putPage');
+
+function _getUser($userid='') {
+    global $request;
+    
+    if (! $userid ) {
+        if (!isset($_SERVER))
+            $_SERVER =& $GLOBALS['HTTP_SERVER_VARS'];
+        if (!isset($_ENV))
+            $_ENV =& $GLOBALS['HTTP_ENV_VARS'];
+        if (isset($_SERVER['REMOTE_USER']))
+            $userid = $_SERVER['REMOTE_USER'];
+        elseif (isset($_ENV['REMOTE_USER']))
+            $userid = $_ENV['REMOTE_USER'];
+        elseif (isset($_SERVER['REMOTE_ADDR']))
+            $userid = $_SERVER['REMOTE_ADDR'];
+        elseif (isset($_ENV['REMOTE_ADDR']))
+            $userid = $_ENV['REMOTE_ADDR'];
+        elseif (isset($GLOBALS['REMOTE_ADDR']))
+            $userid = $GLOBALS['REMOTE_ADDR'];
+    }
+
+    if (ENABLE_USER_NEW) {
+        return WikiUser($userid);
+    } else {
+        return new WikiUser($request, $userid);
+    }
+}
+        
+function putPage($params) {
+    global $request;
+
+    $ParamPageName = $params->getParam(0);
+    $ParamContent = $params->getParam(1);
+    $pagename = short_string_decode($ParamPageName->scalarval());
+    $content = short_string_decode($ParamContent->scalarval());
+    $passwd = '';
+    if (count($params->params) > 2) {
+        $ParamAuthor = $params->getParam(2);
+        $userid = short_string_decode($ParamAuthor->scalarval());
+        if (count($params->params) > 3) {
+            $ParamPassword = $params->getParam(3);
+            $passwd = short_string_decode($ParamPassword->scalarval());
+        }
+    } else {
+    	$userid = $request->_user->_userid;
+    }
+    $request->_user = _getUser($userid);
+    $request->_user->_group = $request->getGroup();
+    $request->_user->AuthCheck($userid, $passwd);
+                                         
+    if (! mayAccessPage ('edit', $pagename)) {
+        return new xmlrpcresp(
+                              new xmlrpcval(
+                                            array('code' => new xmlrpcval(401, "int"), 
+                                                  'version' => new xmlrpcval(0, "int"), 
+                                                  'message' => 
+                                                  short_string("no permission for "
+                                                               .$request->_user->UserName())), 
+                                            "struct"));
+    }
+
+    $now = time();
+    $dbh = $request->getDbh();
+    $page = $dbh->getPage($pagename);
+    $current = $page->getCurrentRevision();
+    $content = trim($content);
+    $version = $current->getVersion();
+    // $version = -1 will force create a new version
+    if ($current->getPackedContent() != $content) {
+        $init_meta = array('ctime' => $now,
+                           'creator' => $userid,
+                           'creator_id' => $userid,
+                           );
+        $version_meta = array('author' => $userid,
+                              'author_id' => $userid,
+                              'markup' => 2.0,
+                              'summary' => isset($summary) ? $summary : _("xml-rpc change"),
+                              'mtime' => $now,
+                              'pagetype' => 'wikitext',
+                              'wikitext' => $init_meta,
+                              );
+        $version++;
+        $res = $page->save($content, $version, $version_meta);
+        if ($res)
+            $message = "Page $pagename version $version created";
+        else
+            $message = "Problem creating version $version of page $pagename";
+    } else {
+    	$res = 0;
+    	$message = $message = "Page $pagename unchanged";
+    }
+    return new xmlrpcresp(new xmlrpcval(array('code'    => new xmlrpcval($res ? 200 : 400, "int"), 
+                                              'version' => new xmlrpcval($version, "int"), 
+                                              'message' => short_string($message)), 
+                                        "struct"));
+}
+
+/**
+ * Publish-Subscribe
+ * Client subscribes to a RecentChanges-like channel, getting a short 
+ * callback notification on every change. Like PageChangeNotification, just shorter 
+ * and more complicated
+ * RSS2 support (not yet), since radio userland's rss-0.92. now called RSS2.
+ * BTW: Radio Userland deprecated this interface.
+ *
+ * boolean wiki.rssPleaseNotify ( notifyProcedure, port, path, protocol, urlList )
+ *   returns: true or false 
+ *
+ * Check of the channel behind the rssurl has a cloud element, 
+ * if the client has a direct IP connection (no NAT),
+ * register the client on the WikiDB notification handler
+ *
+ * http://backend.userland.com/publishSubscribeWalkthrough
+ * http://www.soapware.org/xmlStorageSystem#rssPleaseNotify
+ * http://www.thetwowayweb.com/soapmeetsrss#rsscloudInterface
+ */
+$wiki_dmap['rssPleaseNotify']
+= array('signature'	=> array(array($xmlrpcBoolean, $xmlrpcStruct)),
+        'documentation' => 'RSS2 change notification subscriber channel',
+        'function'	=> 'rssPleaseNotify');
+
+function rssPleaseNotify($params)
+{
+    // register the clients IP
+    return new xmlrpcresp(new xmlrpcval (0, "boolean"));
+}
+
+/*
+ *  String wiki.rssPleaseNotify ( username )
+ *  returns: true or false 
+
+ */
+$wiki_dmap['mailPasswordToUser']
+= array('signature'	=> array(array($xmlrpcBoolean, $xmlrpcString)),
+        'documentation' => 'RSS2 user management helper',
+        'function'	=> 'mailPasswordToUser');
+
+function mailPasswordToUser($params)
+{
+    global $request;
+    $ParamUserid = $params->getParam(0);
+    $userid = short_string_decode($ParamUserid->scalarval());
+    $request->_user = _getUser($userid);
+    //$request->_prefs =& $request->_user->_prefs;
+    $email = $request->getPref('email');
+    $success = 0;
+    if ($email) {
+        $body = WikiURL('') . "\nPassword: " . $request->getPref('passwd');
+        $success = mail($email, "[".WIKI_NAME."} Password Request", 
+                        $body);
+    }
+    return new xmlrpcresp(new xmlrpcval ($success, "boolean"));
+}
+
+/** 
+ * struct titleSearch(String substring [, Integer option = 0])
+ * returns an array of matching pagenames.
+ * TODO: standardize options
+ *
+ * @author: Reini Urban
+ */
+$wiki_dmap['titleSearch']
+= array('signature'     => array(array($xmlrpcArray, $xmlrpcString, $xmlrpcInt)),
+        'documentation' => "Return matching pagenames. 
+Option 1: caseexact, 2: regex, 4: starts_with, 8: exact",
+        'function'      => 'titleSearch');
+
+function titleSearch($params)
+{
+    global $request;
+    $ParamPageName = $params->getParam(0);
+    $searchstring = short_string_decode($ParamPageName->scalarval());
+    if (count($params->params) > 1) {
+        $ParamOption = $params->getParam(1);
+        $option = $ParamOption->scalarval();
+    } else $option = 0;
+    // default option: substring, case-inexact
+
+    $case_exact = $option & 1;
+    $regex      = $option & 2;
+    if (!$regex) {
+        if ($option & 4) { // STARTS_WITH
+            $regex = true;
+            $searchstring = "^".$searchstring;
+        }
+        if ($option & 8) { // EXACT
+            $regex = true;
+            $searchstring = "^".$searchstring."$";
+        }
+    } else {
+    	if ($option & 4 or $option & 8) { 
+		    global $xmlrpcerruser;
+    		return new xmlrpcresp(0, $xmlrpcerruser + 1, "Invalid option");
+    	}
+    }
+    include_once("lib/TextSearchQuery.php");
+    $query = new TextSearchQuery($searchstring, $case_exact, $regex ? 'auto' : 'none');
+    $dbh = $request->getDbh();
+    $iterator = $dbh->titleSearch($query);
+    $pages = array();
+    while ($page = $iterator->next()) {
+        $pages[] = short_string($page->getName());
+    } 
+    return new xmlrpcresp(new xmlrpcval($pages, "array"));
+}
+
  
-// Construct the server instance, and set up the despatch map, which maps
-// the XML-RPC methods onto the wiki functions
+/** 
+ * Construct the server instance, and set up the dispatch map, 
+ * which maps the XML-RPC methods on to the wiki functions.
+ * Provide the "wiki." prefix to each function
+ */
 class XmlRpcServer extends xmlrpc_server
 {
     function XmlRpcServer ($request = false) {
         global $wiki_dmap;
         foreach ($wiki_dmap as $name => $val)
             $dmap['wiki.' . $name] = $val;
-        
+
         $this->xmlrpc_server($dmap, 0 /* delay service*/);
     }
 
@@ -467,7 +717,8 @@ class XmlRpcServer extends xmlrpc_server
         $msg = htmlspecialchars($e->asString());
         // '--' not allowed within xml comment
         $msg = str_replace('--', '&#45;&#45;', $msg);
-        xmlrpc_debugmsg($msg);
+        if (function_exists('xmlrpc_debugmsg'))
+            xmlrpc_debugmsg($msg);
         return true;
     }
 }
@@ -481,4 +732,4 @@ class XmlRpcServer extends xmlrpc_server
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
+?>
\ No newline at end of file
