@@ -1,7 +1,7 @@
 <?php // -*-php-*-
-rcs_id('$Id: SystemInfo.php,v 1.8 2003/02/24 01:36:26 dairiki Exp $');
+rcs_id('$Id: SystemInfo.php,v 1.22 2004/12/26 17:10:44 rurban Exp $');
 /**
- Copyright 1999, 2000, 2001, 2002 $ThePhpWikiProgrammingTeam
+ Copyright (C) 1999, 2000, 2001, 2002 $ThePhpWikiProgrammingTeam
 
  This file is part of PhpWiki.
 
@@ -29,21 +29,21 @@ rcs_id('$Id: SystemInfo.php,v 1.8 2003/02/24 01:36:26 dairiki Exp $');
  *
  * Provide access to phpwiki's lower level system information.
  *
- *   version, CHARSET, pagestats, SERVER_NAME, database, discspace,
+ *   version, charset, pagestats, SERVER_NAME, database, discspace,
  *   cachestats, userstats, linkstats, accessstats, hitstats,
  *   revisionstats, interwikilinks, imageextensions, wikiwordregexp,
- *   availableplugins, downloadurl, ...
+ *   availableplugins, downloadurl  or any other predefined CONSTANT
  *
  * In spirit to http://www.ecyrd.com/JSPWiki/SystemInfo.jsp
  *
  * Todo: Some calculations are heavy (~5-8 secs), so we should cache
  *       the result. In the page or with WikiPluginCached?
  */
-//require_once "lib/WikiPluginCached.php";
 
+require_once "lib/WikiPluginCached.php";
 class WikiPlugin_SystemInfo
-//extends WikiPluginCached
-extends WikiPlugin
+extends WikiPluginCached
+//extends WikiPlugin
 {
     function getPluginType() {
         return PLUGIN_CACHED_HTML;
@@ -58,7 +58,7 @@ extends WikiPlugin
 
     function getVersion() {
         return preg_replace("/[Revision: $]/", '',
-                            "\$Revision: 1.8 $");
+                            "\$Revision: 1.22 $");
     }
 
     function getExpire($dbi, $argarray, $request) {
@@ -68,7 +68,7 @@ extends WikiPlugin
         $loader = new WikiPluginLoader;
         return $loader->expandPI('<?plugin SystemInfo '
                                  . WikiPluginCached::glueArgs($argarray) // all
-                                 . ' ?>', $request, $basepage);
+                                 . ' ?>', $request, $this, $basepage);
     }
     /*
     function getDefaultArguments() {
@@ -138,19 +138,9 @@ extends WikiPlugin
     }
     function pagestats() {
         global $request;
-        $e = 0;
-        $a = 0;
         $dbi = $request->getDbh();
-        $include_empty = true;
-        $iter = $dbi->getAllPages($include_empty);
-        while ($page = $iter->next())
-            $e++;
-        $s  = sprintf(_("%d pages"), $e);
-        $include_empty = false;
-        $iter = $dbi->getAllPages($include_empty);
-        while ($page = $iter->next())
-            $a++;
-        $s  .= ", " . sprintf(_("%d not-empty pages"), $a);
+        $s  = sprintf(_("%d pages"), $dbi->numPages(true));
+        $s  .= ", " . sprintf(_("%d not-empty pages"), $dbi->numPages(false));
         // more bla....
         // $s  .= ", " . sprintf(_("earliest page from %s"), $earliestdate);
         // $s  .= ", " . sprintf(_("latest page from %s"), $latestdate);
@@ -185,13 +175,16 @@ extends WikiPlugin
         // $s  .= ", " . sprintf(_("%d externally authenticated users"), $extauth); // query AuthDB?
         return $s;
     }
+
     //only from logging info possible. = hitstats per time.
     // total hits per day/month/year
     // view/edit rate
+    // TODO: see WhoIsOnline hit stats, and sql accesslogs
     function accessstats() {
         $s  = _("not yet");
         return $s;
     }
+
     // only absolute numbers, not for any time interval. see accessstats
     //  some useful number derived from the curve of the hit stats.
     //  total, max, mean, median, stddev;
@@ -262,7 +255,7 @@ extends WikiPlugin
         }
         return 'not yet';
     }
-    // size of databases/files/cvs are possible plus the known size of the app.
+    // Size of databases/files/cvs are possible plus the known size of the app.
     // Todo: cache this costly operation!
     function discspace() {
         global $DBParams;
@@ -290,13 +283,13 @@ extends WikiPlugin
     }
 
     function inlineimages () {
-        return implode(' ', explode('|', $GLOBALS['InlineImages']));
+        return implode(' ', explode('|', INLINE_IMAGES));
     }
     function wikinameregexp () {
         return $GLOBALS['WikiNameRegexp'];
     }
     function allowedprotocols () {
-        return implode(' ', explode('|', $GLOBALS['AllowedProtocols']));
+        return implode(' ', explode('|', ALLOWED_PROTOCOLS));
     }
     function available_plugins () {
         $fileset = new FileSet(FindFile('lib/plugin'), '*.php');
@@ -309,22 +302,7 @@ extends WikiPlugin
                                       $list));
     }
     function supported_languages () {
-        $available_languages = array('en');
-        $dir_root = 'locale/';
-        if (defined('PHPWIKI_DIR'))
-            $dir_root = PHPWIKI_DIR . '/$dir_root';
-        $dir = dir($dir_root);
-        if ($dir) {
-            while($entry = $dir->read()) {
-                if (is_dir($dir_root . $entry)
-                    && (substr($entry, 0, 1) != '.')
-                    && $entry != 'po'
-                    && $entry != 'CVS') {
-                    array_push($available_languages, $entry);
-                }
-            }
-            $dir->close();
-        }
+        $available_languages = listAvailableLanguages();
         natcasesort($available_languages);
 
         return sprintf(_("Total of %d languages: "),
@@ -337,27 +315,13 @@ extends WikiPlugin
     }
 
     function supported_themes () {
-        global $Theme;
-        $available_themes = array();
-        $dir_root = 'themes/';
-        if (defined('PHPWIKI_DIR'))
-            $dir_root = PHPWIKI_DIR . '/$dir_root';
-        $dir = dir($dir_root);
-        if ($dir) {
-            while($entry = $dir->read()) {
-                if (is_dir($dir_root.$entry)
-                    && (substr($entry,0,1) != '.')
-                    && $entry!='CVS') {
-                    array_push($available_themes, $entry);
-                }
-            }
-            $dir->close();
-        }
+        global $WikiTheme;
+        $available_themes = listAvailableThemes();
         natcasesort($available_themes);
         return sprintf(_("Total of %d themes: "), count($available_themes))
             . implode(', ',$available_themes) . ". "
-            . sprintf(_("Current theme: '%s'"), $Theme->_name)
-            . ((THEME != $Theme->_name)
+            . sprintf(_("Current theme: '%s'"), $WikiTheme->_name)
+            . ((THEME != $WikiTheme->_name)
                ? ". " . sprintf(_("Default theme: '%s'"), THEME)
                : '');
     }
@@ -365,15 +329,16 @@ extends WikiPlugin
     function call ($arg, &$availableargs) {
         if (!empty($availableargs[$arg]))
             return $availableargs[$arg]();
-        elseif (method_exists($this, $arg)) // any defined SystemInfo->method()system
+        elseif (method_exists($this, $arg)) // any defined SystemInfo->method()
             return call_user_func_array(array(&$this, $arg), '');
-        elseif (defined($arg) && $arg != 'ADMIN_PASSWD') // any defined constant
+        elseif (defined($arg) && // any defined constant
+                !in_array($arg,array('ADMIN_PASSWD','DATABASE_DSN','DBAUTH_AUTH_DSN')))
             return constant($arg);
         else
             return $this->error(sprintf(_("unknown argument '%s' to SystemInfo"), $arg));
     }
 
-    function run($dbi, $argstr, $request, $basepage) {
+    function run($dbi, $argstr, &$request, $basepage) {
         // don't parse argstr for name=value pairs. instead we use just 'name'
         //$args = $this->getArgs($argstr, $request);
         $args['seperator'] = ' ';
@@ -442,24 +407,6 @@ extends WikiPlugin
     }
 }
 
-/* // autolisp stdlib
-;;; Median of the sorted list of numbers. 50% is above and 50% below
-;;; "center of a distribution"
-;;; Ex: (std-median (std-make-list 100 std-%random)) => 0.5 +- epsilon
-;;;     (std-median (std-make-list 100 (lambda () (std-random 10))))
-;;;       => 4.0-5.0 [0..9]
-;;;     (std-median (std-make-list 99  (lambda () (std-random 10))))
-;;;       => 4-5
-;;;     (std-median '(0 0 2 4 12))      => 2
-;;;     (std-median '(0 0 4 12))        => 2.0
-(defun STD-MEDIAN (numlst / l)
-  (setq numlst (std-sort numlst '<))            ; don't remove duplicates
-  (if (= 0 (rem (setq l (length numlst)) 2))    ; if even length
-    (* 0.5 (+ (nth (/ l 2) numlst)              ; force float!
-              (nth (1- (/ l 2)) numlst)))       ; fixed by Serge Pashkov
-    (nth (/ l 2) numlst)))
-
-*/
 function median($hits) {
     sort($hits);
     reset($hits);
@@ -488,29 +435,9 @@ function gensym($prefix = "_gensym") {
     return $prefix . $i;
 }
 
-/* // autolisp stdlib
-(defun STD-STANDARD-DEVIATION (numlst / n _dev_m r)
-  (setq n      (length numlst)
-        _dev_m (std-mean numlst)
-        r      (mapcar (function (lambda (x) (std-sqr (- x _dev_m)))) numlst))
-  (sqrt (* (std-mean r) (/ n (float (- n 1))))))
-*/
-/*
-function stddev(&$hits, $total=false) {
-    $n = count($hits);
-    if (!$total) $total = array_reduce($hits, 'rsum');
-    $mean = gensym("_mean");
-    $GLOBALS[$mean] = $total / $n;
-    $cb = "global ${$mean}; return (\$i-${$mean})*(\$i-${$mean});";
-    $r = array_map(create_function('$i',"global ${$mean}; return (\$i-${$mean})*(\$i-${$mean});"),$hits);
-    unset($GLOBALS[$mean]);
-    return (float) sqrt(mean($r,$total) * ($n / (float)($n -1)));
-}
-*/
 function stddev(&$hits, $total = false) {
     $n = count($hits);
-    if (!$total)
-        $total = array_reduce($hits, 'rsum');
+    if (!$total) $total = array_reduce($hits, 'rsum');
     $GLOBALS['mean'] = $total / $n;
     $r = array_map(create_function('$i', 'global $mean; return ($i-$mean)*($i-$mean);'),
                    $hits);
@@ -519,6 +446,64 @@ function stddev(&$hits, $total = false) {
 }
 
 // $Log: SystemInfo.php,v $
+// Revision 1.22  2004/12/26 17:10:44  rurban
+// just docs or whitespace
+//
+// Revision 1.21  2004/11/26 18:39:02  rurban
+// new regex search parser and SQL backends (90% complete, glob and pcre backends missing)
+//
+// Revision 1.20  2004/11/20 11:28:49  rurban
+// fix a yet unused PageList customPageListColumns bug (merge class not decl to _types)
+// change WantedPages to use PageList
+// change WantedPages to print the list of referenced pages, not just the count.
+//   the old version was renamed to WantedPagesOld
+//   fix and add handling of most standard PageList arguments (limit, exclude, ...)
+// TODO: pagename sorting, dumb/WantedPagesIter and SQL optimization
+//
+// Revision 1.19  2004/06/19 11:49:42  rurban
+// dont print db passwords
+//
+// Revision 1.18  2004/06/14 11:31:39  rurban
+// renamed global $Theme to $WikiTheme (gforge nameclash)
+// inherit PageList default options from PageList
+//   default sortby=pagename
+// use options in PageList_Selectable (limit, sortby, ...)
+// added action revert, with button at action=diff
+// added option regex to WikiAdminSearchReplace
+//
+// Revision 1.17  2004/05/08 14:06:13  rurban
+// new support for inlined image attributes: [image.jpg size=50x30 align=right]
+// minor stability and portability fixes
+//
+// Revision 1.16  2004/05/06 20:30:47  rurban
+// revert and removed some comments
+//
+// Revision 1.15  2004/05/03 11:40:42  rurban
+// put listAvailableLanguages() and listAvailableThemes() from SystemInfo and
+// UserPreferences into Themes.php
+//
+// Revision 1.14  2004/04/19 23:13:04  zorloc
+// Connect the rest of PhpWiki to the IniConfig system.  Also the keyword regular expression is not a config setting
+//
+// Revision 1.13  2004/04/18 01:11:52  rurban
+// more numeric pagename fixes.
+// fixed action=upload with merge conflict warnings.
+// charset changed from constant to global (dynamic utf-8 switching)
+//
+// Revision 1.12  2004/03/14 16:26:21  rurban
+// copyright line
+//
+// Revision 1.11  2004/03/13 19:24:21  rurban
+// fixed supported_languages() and supported_themes()
+//
+// Revision 1.10  2004/03/08 18:17:10  rurban
+// added more WikiGroup::getMembersOf methods, esp. for special groups
+// fixed $LDAP_SET_OPTIONS
+// fixed _AuthInfo group methods
+//
+// Revision 1.9  2004/02/17 12:11:36  rurban
+// added missing 4th basepage arg at plugin->run() to almost all plugins. This caused no harm so far, because it was silently dropped on normal usage. However on plugin internal ->run invocations it failed. (InterWikiSearch, IncludeSiteMap, ...)
+//
 // Revision 1.8  2003/02/24 01:36:26  dairiki
 // Don't use PHPWIKI_DIR unless it's defined.
 // (Also typo/bugfix in SystemInfo plugin.)
