@@ -1,4 +1,4 @@
-<?php rcs_id('$Id: ziplib.php,v 1.31 2003/11/09 01:24:13 carstenklapp Exp $');
+<?php rcs_id('$Id: ziplib.php,v 1.44 2005/01/31 00:28:48 rurban Exp $');
 
 /**
  * GZIP stuff.
@@ -20,12 +20,14 @@ function gzip_cleanup () {
 function gzip_tempnam () {
     global $gzip_tmpfile;
     
-    if (!$gzip_tmpfile)
-        {
-            //FIXME: does this work on non-unix machines?
+    if (!$gzip_tmpfile) {
+        //FIXME: does this work on non-unix machines?
+        if (is_writable("/tmp"))
             $gzip_tmpfile = tempnam("/tmp", "wkzip");
-            register_shutdown_function("gzip_cleanup");
-        }
+        else
+            $gzip_tmpfile = tempnam(TEMP_DIR, "wkzip");
+        register_shutdown_function("gzip_cleanup");
+    }
     return $gzip_tmpfile;
 }
 
@@ -36,7 +38,7 @@ function gzip_compress ($data) {
     gzwrite($fp, $data, strlen($data));
     if (!gzclose($fp)) {
         trigger_error(sprintf("%s failed", 'gzclose'), E_USER_ERROR);
-	}
+    }
 /* ---- Original code ----  
 	$size = filesize($filename);
     if (!($fp = fopen($filename, "rb"))) {
@@ -48,12 +50,12 @@ function gzip_compress ($data) {
         trigger_error(sprintf("%s failed", 'fclose'), E_USER_ERROR);
 */
 // -- FIX -------------
-	$z = NULL;
+    $z = NULL;
     if (!($fp = fopen($filename,"rb"))) {
         trigger_error(sprintf("%s failed", 'fopen'), E_USER_ERROR);
     }
     while(!feof($fp)) {
-        $z.= fread($fp,1024);
+        $z .= fread($fp,1024);
     }
     if (!fclose($fp))
         trigger_error(sprintf("%s failed", 'fclose'), E_USER_ERROR);
@@ -166,7 +168,7 @@ function zip_deflate ($content)
     if (function_exists('gzencode'))
         $z = gzencode($content);
     else
-    $z = gzip_compress($content);
+        $z = gzip_compress($content);
     
     // Suck OS type byte from gzip header. FIXME: this smells bad.
     
@@ -263,9 +265,9 @@ class ZipWriter
         $this->dir = "";		// "Central directory block"
         $this->offset = 0;		// Current file position.
         
-    $zipname = addslashes($zipname);
-    header("Content-Type: application/zip; name=\"$zipname\"");
-    header("Content-Disposition: attachment; filename=\"$zipname\"");
+        $zipname = addslashes($zipname);
+        header("Content-Type: application/zip; name=\"$zipname\"");
+        header("Content-Disposition: attachment; filename=\"$zipname\"");
     }
     
   function addRegularFile ($filename, $content, $attrib = false) {
@@ -273,29 +275,25 @@ class ZipWriter
           $attrib = array();
       
       $size = strlen($content);
-      if (function_exists('gzopen'))
-          {
-              list ($data, $crc32, $os_type) = zip_deflate($content);
-              if (strlen($data) < $size)
-                  {
-                      $content = $data;	// Use compressed data.
-                      $comp_type = ZIP_DEFLATE;
-                  }
-              else
-                  unset($crc32);	// force plain store.
-          }
-      else
-          {
-              // Punt:
-              $os_type = 0;     // 0 = FAT --- hopefully this is good enough.
-              /* (Another choice might be 3 = Unix) */
+      if (function_exists('gzopen')) {
+          list ($data, $crc32, $os_type) = zip_deflate($content);
+          if (strlen($data) < $size) {
+              $content = $data;	// Use compressed data.
+              $comp_type = ZIP_DEFLATE;
           }
+          else
+              unset($crc32);	// force plain store.
+      }
+      else  {
+          // Punt:
+          $os_type = 3;     // 0 = FAT --- hopefully this is good enough.
+          /* (Another choice might be 3 = Unix) */
+      }
 
-      if (!isset($crc32))
-          {
-              $comp_type = ZIP_STORE;
-              $crc32 = zip_crc32($content);
-          }
+      if (!isset($crc32)) {
+          $comp_type = ZIP_STORE;
+          $crc32 = zip_crc32($content);
+      }
       
       if (!empty($attrib['write_protected']))
           $atx = (0100444 << 16) | 1; // S_IFREG + read permissions to
@@ -319,7 +317,7 @@ class ZipWriter
       $head = pack("vvvvvVVVvv",
                    20,	// Version needed to extract (FIXME: is this right?)
                    0,	// Gen purp bit flag
-		 $comp_type,
+                   $comp_type,
                    $mod_time,
                    $mod_date,
                    $crc32,
@@ -424,13 +422,12 @@ class ZipReader
       //FIXME: we should probably check $req_version.
       $attrib['mtime'] = dostime2unixtime($mod_date, $mod_time);
       
-      if ($magic != ZIP_LOCHEAD_MAGIC)
-          {
-              if ($magic != ZIP_CENTHEAD_MAGIC)
-                  // FIXME: better message?
-                  ExitWiki(sprintf("Bad header type: %s", $magic));
-              return $this->done();
-          }
+      if ($magic != ZIP_LOCHEAD_MAGIC) {
+          if ($magic != ZIP_CENTHEAD_MAGIC)
+              // FIXME: better message?
+              ExitWiki(sprintf("Bad header type: %s", $magic));
+          return $this->done();
+      }
       if (($flags & 0x21) != 0)
           ExitWiki("Encryption and/or zip patches not supported.");
       if (($flags & 0x08) != 0)
@@ -443,16 +440,14 @@ class ZipReader
       
       $data = $this->_read($comp_size);
       
-      if ($comp_type == ZIP_DEFLATE)
-          {
-              $data = zip_inflate($data, $crc32, $uncomp_size);
-          }
-      else if ($comp_type == ZIP_STORE)
-          {
-              $crc = zip_crc32($data);
-              if ($crc32 != $crc)
-                  ExitWiki(sprintf("CRC mismatch %x != %x", $crc, $crc32));
-          }
+      if ($comp_type == ZIP_DEFLATE) {
+          $data = zip_inflate($data, $crc32, $uncomp_size);
+      }
+      else if ($comp_type == ZIP_STORE) {
+          $crc = zip_crc32($data);
+          if ($crc32 != $crc)
+              ExitWiki(sprintf("CRC mismatch %x != %x", $crc, $crc32));
+      }
       else
           ExitWiki(sprintf("Compression method %s unsupported",
                            $comp_method));
@@ -477,15 +472,14 @@ function QuotedPrintableEncode ($string)
 {
     // Quote special characters in line.
     $quoted = "";
-    while ($string)
-        {
-            // The complicated regexp is to force quoting of trailing spaces.
-            preg_match('/^([ !-<>-~]*)(?:([!-<>-~]$)|(.))/s', $string, $match);
-            $quoted .= $match[1] . $match[2];
-            if (!empty($match[3]))
-                $quoted .= sprintf("=%02X", ord($match[3]));
-            $string = substr($string, strlen($match[0]));
-        }
+    while ($string) {
+        // The complicated regexp is to force quoting of trailing spaces.
+        preg_match('/^([ !-<>-~]*)(?:([!-<>-~]$)|(.))/s', $string, $match);
+        $quoted .= $match[1] . $match[2];
+        if (!empty($match[3]))
+            $quoted .= sprintf("=%02X", ord($match[3]));
+        $string = substr($string, strlen($match[0]));
+    }
     // Split line.
     // This splits the line (preferably after white-space) into lines
     // which are no longer than 76 chars (after adding trailing '=' for
@@ -507,13 +501,12 @@ function MimeContentTypeHeader ($type, $subtype, $params)
 {
     $header = "Content-Type: $type/$subtype";
     reset($params);
-    while (list($key, $val) = each($params))
-        {
-            //FIXME:  what about non-ascii printables in $val?
-            if (!preg_match('/^' . MIME_TOKEN_REGEXP . '$/', $val))
-                $val = '"' . addslashes($val) . '"';
-            $header .= ";\r\n  $key=$val";
-        }
+    while (list($key, $val) = each($params)) {
+        //FIXME:  what about non-ascii printables in $val?
+        if (!preg_match('/^' . MIME_TOKEN_REGEXP . '$/', $val))
+            $val = '"' . addslashes($val) . '"';
+        $header .= ";\r\n  $key=$val";
+    }
     return "$header\r\n";
 }
 
@@ -561,7 +554,6 @@ function MimeifyPageRevision ($revision) {
                     'author'       => $revision->get('author'),
                     'version'      => $revision->getVersion(),
                     'lastmodified' => $revision->get('mtime'));
-
     
     if ($page->get('mtime'))
         $params['created'] = $page->get('mtime');
@@ -569,19 +561,30 @@ function MimeifyPageRevision ($revision) {
         $params['flags'] = 'PAGE_LOCKED';
     if ($revision->get('author_id'))
         $params['author_id'] = $revision->get('author_id');
-    if ($revision->get('markup'))
+    if ($revision->get('markup')) // what is the default? we must use 1
         $params['markup'] = $revision->get('markup');
     if ($revision->get('summary'))
         $params['summary'] = $revision->get('summary');
     if ($page->get('hits'))
         $params['hits'] = $page->get('hits');
+    if ($page->get('owner'))
+        $params['owner'] = $page->get('owner');
+    if ($page->get('perm') and class_exists('PagePermission')) {
+        $acl = getPagePermissions($page);
+        $params['acl'] = $acl->asAclLines();
+        //TODO: convert to multiple lines? acl-view => groups,...; acl-edit => groups,... 
+    }
 
-    $params['charset'] = CHARSET;
+    $params['charset'] = $GLOBALS['charset'];
 
     // Non-US-ASCII is not allowed in Mime headers (at least not without
     // special handling) --- so we urlencode all parameter values.
     foreach ($params as $key => $val)
         $params[$key] = rawurlencode($val);
+    if (isset($params['acl'])) 
+        // default: "view:_EVERY; edit:_AUTHENTICATED; create:_AUTHENTICATED,_BOGOUSER; ".
+        //          "list:_EVERY; remove:_ADMIN,_OWNER; change:_ADMIN,_OWNER; dump:_EVERY; "
+        $params['acl'] = str_replace(array("%3A","%3B%20","%2C"),array(":","; ",","),$params['acl']);
     
     $out = MimeContentTypeHeader('application', 'x-phpwiki', $params);
     $out .= sprintf("Content-Transfer-Encoding: %s\r\n",
@@ -604,31 +607,30 @@ function MimeifyPageRevision ($revision) {
  */
 function ParseRFC822Headers (&$string)
 {
-    if (preg_match("/^From (.*)\r?\n/", $string, $match))
-        {
-            $headers['from '] = preg_replace('/^\s+|\s+$/', '', $match[1]);
-            $string = substr($string, strlen($match[0]));
-        }
+    if (preg_match("/^From (.*)\r?\n/", $string, $match)) {
+        $headers['from '] = preg_replace('/^\s+|\s+$/', '', $match[1]);
+        $string = substr($string, strlen($match[0]));
+    }
     
     while (preg_match('/^([!-9;-~]+) [ \t]* : [ \t]* '
                       . '( .* \r?\n (?: [ \t] .* \r?\n)* )/x',
                       $string, $match))
-        {
-            $headers[strtolower($match[1])]
-                = preg_replace('/^\s+|\s+$/', '', $match[2]);
-            $string = substr($string, strlen($match[0]));
-        }
+    {
+        $headers[strtolower($match[1])]
+            = preg_replace('/^\s+|\s+$/', '', $match[2]);
+        $string = substr($string, strlen($match[0]));
+    }
     
     if (empty($headers))
         return false;
-    
-    if (! preg_match("/^\r?\n/", $string, $match))
-        {
+
+    if (strlen($string) > 0) {
+        if (! preg_match("/^\r?\n/", $string, $match))  {
             // No blank line after headers.
             return false;
         }
-    
-    $string = substr($string, strlen($match[0]));
+        $string = substr($string, strlen($match[0]));
+    }
     
     return $headers;
 }
@@ -652,18 +654,17 @@ function ParseMimeContentType ($string)
     $param = array();
     while (preg_match('/^;\s*(' . MIME_TOKEN_REGEXP . ')\s*=\s*'
                       . '(?:(' . MIME_TOKEN_REGEXP . ')|"((?:[^"\\\\]|\\.)*)") \s*/sx',
-                      $string, $match))
-        {
-            //" <--kludge for brain-dead syntax coloring
-            if (strlen($match[2]))
-                $val = $match[2];
-            else
-                $val = preg_replace('/[\\\\](.)/s', '\\1', $match[3]);
-            
-            $param[strtolower($match[1])] = $val;
-            
-            $string = substr($string, strlen($match[0]));
-        }
+                      $string, $match)) {
+        //" <--kludge for brain-dead syntax coloring
+        if (strlen($match[2]))
+            $val = $match[2];
+        else
+            $val = preg_replace('/[\\\\](.)/s', '\\1', $match[3]);
+        
+        $param[strtolower($match[1])] = $val;
+        
+        $string = substr($string, strlen($match[0]));
+    }
     
     return array($type, $subtype, $param);
 }
@@ -677,19 +678,19 @@ function ParseMimeMultipart($data, $boundary)
     
     while (preg_match("/^(|.*?\n)--$boundary((?:--)?)[^\n]*\n/s",
                       $data, $match))
-        {
-            $data = substr($data, strlen($match[0]));
-            if ( ! isset($parts) )
-                $parts = array();  // First time through: discard leading chaff
-            else {
-                if ($content = ParseMimeifiedPages($match[1]))
-                    for (reset($content); $p = current($content); next($content))
-                        $parts[] = $p;
-            }
-            
-            if ($match[2])
-                return $parts;	// End boundary found.
+    {
+        $data = substr($data, strlen($match[0]));
+        if ( ! isset($parts) )
+            $parts = array();  // First time through: discard leading chaff
+        else {
+            if ($content = ParseMimeifiedPages($match[1]))
+                for (reset($content); $p = current($content); next($content))
+                    $parts[] = $p;
         }
+            
+        if ($match[2])
+            return $parts;	// End boundary found.
+    }
     ExitWiki("No end boundary?");
 }
 
@@ -697,24 +698,45 @@ function GenerateFootnotesFromRefs($params)
 {
     $footnotes = array();
     reset($params);
-    while (list($p, $reference) = each($params))
-        {
+    while (list($p, $reference) = each($params)) {
             if (preg_match('/^ref([1-9][0-9]*)$/', $p, $m))
                 $footnotes[$m[1]] = sprintf(_("[%d] See [%s]"),
                                             $m[1], rawurldecode($reference));
-        }
+    }
     
-    if (sizeof($footnotes) > 0)
-        {
-            ksort($footnotes);
-            return "-----\n"
-                . "!" ._("References") . "\n"
-                . join("\n%%%\n", $footnotes) . "\n";
-        }
-    else
+    if (sizeof($footnotes) > 0) {
+        ksort($footnotes);
+        return "-----\n"
+            . "!" ._("References") . "\n"
+            . join("\n%%%\n", $footnotes) . "\n";
+    } else
         return "";
 }
 
+// counterpart to $acl->asAclLines() and rawurl undecode
+// default: "view:_EVERY; edit:_AUTHENTICATED; create:_AUTHENTICATED,_BOGOUSER; ".
+//          "list:_EVERY; remove:_ADMIN,_OWNER; change:_ADMIN,_OWNER; dump:_EVERY; "
+function ParseMimeifiedPerm($string) {
+    if (!class_exists('PagePermission')) {
+        return '';
+    }
+    $hash = array();
+    foreach (split(";",trim($string)) as $accessgroup) {
+        list($access,$groupstring) = split(":",trim($accessgroup));
+        $access = trim($access);
+        $groups = split(",",trim($groupstring));
+        foreach ($groups as $group) {
+            $group = trim($group);
+            $bool = (boolean) (substr($group,0,1) != '-');
+            if (substr($group,0,1) == '-' or substr($group,0,1) == '+')
+                $group = substr($group,1);
+            $hash[$access][$group] = $bool;
+        }
+    }
+    $perm = new PagePermission($hash);
+    $perm->sanify();
+    return serialize($perm->perm);
+}
 
 // Convert references in meta-data to footnotes.
 // Only zip archives generated by phpwiki 1.2.x or earlier should have
@@ -748,7 +770,9 @@ function ParseMimeifiedPages ($data)
     $page        = array();
     $pagedata    = array();
     $versiondata = array();
-    
+    $pagedata['date'] = strtotime($headers['date']);
+
+    //DONE: support owner and acl
     foreach ($params as $key => $value) {
         if (empty($value))
             continue;
@@ -762,10 +786,17 @@ function ParseMimeifiedPages ($data)
             if (preg_match('/PAGE_LOCKED/', $value))
                 $pagedata['locked'] = 'yes';
             break;
+        case 'owner':
         case 'created':
         case 'hits':
             $pagedata[$key] = $value;
             break;
+        case 'acl':
+        case 'perm':
+            if (class_exists('PagePermission')) {
+                $pagedata['perm'] = ParseMimeifiedPerm($value);
+            }
+            break;
         case 'lastmodified':
             $versiondata['mtime'] = $value;
             break;
@@ -802,6 +833,57 @@ function ParseMimeifiedPages ($data)
     return array($page);
 }
 
+// $Log: ziplib.php,v $
+// Revision 1.44  2005/01/31 00:28:48  rurban
+// fix bug 1044945: pgsrc upgrade problem if mime wo/body
+//
+// Revision 1.43  2005/01/25 08:00:09  rurban
+// use unix type to support subdirs with forward slash and long filenames
+//
+// Revision 1.42  2004/11/16 16:17:51  rurban
+// support ENABLE_PAGEPERM=false mime load/save
+//
+// Revision 1.41  2004/11/01 10:43:58  rurban
+// seperate PassUser methods into seperate dir (memory usage)
+// fix WikiUser (old) overlarge data session
+// remove wikidb arg from various page class methods, use global ->_dbi instead
+// ...
+//
+// Revision 1.40  2004/06/19 12:32:37  rurban
+// new TEMP_DIR for ziplib
+//
+// Revision 1.39  2004/06/08 10:54:47  rurban
+// better acl dump representation, read back acl and owner
+//
+// Revision 1.38  2004/06/08 10:05:11  rurban
+// simplified admin action shortcuts
+//
+// Revision 1.37  2004/06/07 22:28:04  rurban
+// add acl field to mimified dump
+//
+// Revision 1.36  2004/06/07 19:50:40  rurban
+// add owner field to mimified dump
+//
+// Revision 1.35  2004/05/02 21:26:38  rurban
+// limit user session data (HomePageHandle and auth_dbi have to invalidated anyway)
+//   because they will not survive db sessions, if too large.
+// extended action=upgrade
+// some WikiTranslation button work
+// revert WIKIAUTH_UNOBTAINABLE (need it for main.php)
+// some temp. session debug statements
+//
+// Revision 1.34  2004/04/18 01:11:52  rurban
+// more numeric pagename fixes.
+// fixed action=upload with merge conflict warnings.
+// charset changed from constant to global (dynamic utf-8 switching)
+//
+// Revision 1.33  2004/04/12 13:04:50  rurban
+// added auth_create: self-registering Db users
+// fixed IMAP auth
+// removed rating recommendations
+// ziplib reformatting
+//
+
 // Local Variables:
 // mode: php
 // tab-width: 8
@@ -809,4 +891,4 @@ function ParseMimeifiedPages ($data)
 // c-hanging-comment-ender-p: nil
 // indent-tabs-mode: nil
 // End:   
-?>
+?>
\ No newline at end of file
